/*
/ Classdef.h
/ class definitions for LibreAtlas
/
/ version 1.0, 2012 May 16
/
/ Author: Sandro Furieri a-furieri@lqt.it
/
/ Copyright (C) 2012  Alessandro Furieri
/
/    This program is free software: you can redistribute it and/or modify
/    it under the terms of the GNU General Public License as published by
/    the Free Software Foundation, either version 3 of the License, or
/    (at your option) any later version.
/
/    This program is distributed in the hope that it will be useful,
/    but WITHOUT ANY WARRANTY; without even the implied warranty of
/    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
/    GNU General Public License for more details.
/
/    You should have received a copy of the GNU General Public License
/    along with this program.  If not, see <http://www.gnu.org/licenses/>.
/
*/

#include "wx/wx.h"

#include "config.h"

#ifdef SPATIALITE_AMALGAMATION
#include <spatialite/sqlite3.h>
#else
#include <sqlite3.h>
#endif

#include <spatialite/gaiaaux.h>
#include <spatialite/gaiaexif.h>
#include <spatialite/gaiageo.h>
#include <spatialite.h>

#include <rasterlite.h>

enum
{
// control IDs for main window controls
  ID_Political = 1,
  ID_Satellite,
  ID_Relief,
  ID_Bathymetry,
  ID_Temperature,
  ID_Rain,
  ID_Month,
  ID_January,
  ID_April,
  ID_July,
  ID_October,
  ID_Center,
  ID_BestZoom,
  ID_Identify,
  ID_ZoomIn,
  ID_ZoomOut,
  ID_Pan,
  ID_Graticule,
  ID_Distance,
};

enum
{
// map layer IDs
  ATLAS_COUNTRIES = 1,
  ATLAS_ADMIN_AREAS_1,
  ATLAS_ADMIN_AREAS_2,
  ATLAS_ADMIN_AREAS_3,
  ATLAS_CULTURAL_HERITAGE,
  ATLAS_NAMES,
  ATLAS_NATURAL_RESERVES,
  ATLAS_LAKES,
  ATLAS_RIVERS,
  ATLAS_HIGHWAYS,
  ATLAS_RAILWAYS,
  ATLAS_RAILWAY_STATIONS,
  ATLAS_LAND_AREAS,
  ATLAS_MARINE_AREAS,
  ATLAS_TOWN_AREAS_MINOR,
  ATLAS_TOWN_AREAS_MEDIUM,
  ATLAS_TOWN_AREAS_MAJOR,
  ATLAS_TOWNS_MINOR,
  ATLAS_TOWNS_MEDIUM,
  ATLAS_TOWNS_MAJOR,
  ATLAS_GEOGRAPHIC_LINES,
  ATLAS_GRATICULES,
  ATLAS_SATELLITE,
  ATLAS_RELIEF,
  ATLAS_BATHYMETRY,
  ATLAS_TEMPERATURE_JAN,
  ATLAS_TEMPERATURE_APR,
  ATLAS_TEMPERATURE_JUL,
  ATLAS_TEMPERATURE_OCT,
  ATLAS_RAIN_JAN,
  ATLAS_RAIN_APR,
  ATLAS_RAIN_JUL,
  ATLAS_RAIN_OCT,
  ATLAS_NONE,
};

enum
{
// control IDs for timers
  ID_MOUSE_TIMER = 20000,
  ID_WHEEL_TIMER,
  ID_DYNAMIC_TIMER,
};

class MyApp:public wxApp
{
//
// the main APP
//
  virtual bool OnInit();
};

class MyTipWindow:public wxWindow
{
//
// a TipWindow used for on-the-fly Identify
//
private:
  class MyMapView * Parent;
  wxString Text;
  int MouseX;
  int MouseY;
  int Category;
public:
    MyTipWindow(MyMapView * parent, wxString & text, int x, int y,
                int category);
    virtual ~ MyTipWindow()
  {;
  }
  bool IsToBeErased(int x, int y);
  void OnPaint(wxPaintEvent & event);
  void OnEraseBackground(wxEraseEvent & WXUNUSED(event))
  {;
  }
  void OnMouseEvent(wxMouseEvent & event);
};

class GreatCirclePoint
{
// a class representing a GreatCircleArc point
private:
  double X;
  double Y;
public:
    GreatCirclePoint()
  {
    X = 0.0;
    Y = 0.0;
  }
   ~GreatCirclePoint()
  {;
  }
  void Set(double x, double y)
  {
    X = x;
    Y = y;
  }
  double GetX()
  {
    return X;
  }
  double GetY()
  {
    return Y;
  }
};

class GreatCirclePath
{
// a class representing a GreatCircleArc
private:
  GreatCirclePoint * Points;
  int NumPoints;
  bool CrossesDateLine;
  double DateLineY;
  int BreakIndex;
  void ComputePoint(int from, int to, int mid);
  void ExpandPoints();
  void InterpolatePoints();
public:
    GreatCirclePath(double x1, double y1, double x2, double y2);
   ~GreatCirclePath();
  int GetNumPoints()
  {
    return NumPoints;
  }
  int GetNumPointsOne()
  {
    return BreakIndex + 1;
  }
  int GetNumPointsTwo()
  {
    return NumPoints - BreakIndex;
  }
  GreatCirclePoint *GetPoint(int n);
  bool DoesCrossesDateLine()
  {
    return CrossesDateLine;
  }
  double GetDateLineY()
  {
    return DateLineY;
  }
};

class MyMapView:public wxPanel
{
//
// a panel used  to show the Map
//
private:
  class MyFrame * MainFrame;
  wxBitmap MapBitmap;
  wxBitmap ScreenBitmap;
  wxBitmap DynamicMapBitmap;
  int BitmapWidth;
  int BitmapHeight;
  wxCursor CursorCross;
  wxCursor CursorHand;
  MyTipWindow *MapTip;
  bool ValidMap;
  int FrameWidth;
  int FrameHeight;
  double FrameCenterX;
  double FrameCenterY;
  double FrameExtentX;
  double FrameExtentY;
  double FrameMinX;
  double FrameMinY;
  double FrameMaxX;
  double FrameMaxY;
  double PixelRatio;
  int DragStartX;
  int DragStartY;
  int LastDragX;
  int LastDragY;
  double DistanceX1;            // Distance arc: point 1 X
  double DistanceY1;            // Distance arc: point 1 Y
  double DistanceX2;            // Distance arc: point 2 X
  double DistanceY2;            // Distance arc: point 2 Y
  wxBitmap Pin;                 // Distance point marker
  int CurrentScale;
  gaiaGeomCollPtr CurrentGeometry;
  gaiaGeomCollPtr DistancePath;
// Timers
  int MouseLastX;
  int MouseLastY;
  wxTimer *TimerMouseMove;
  int WheelTics;
  wxTimer *TimerMouseWheel;
  int DynamicPhase;
  wxTimer *TimerDynamic;
  double InternalRound(double value);
  int IntRound(double value);
public:
    MyMapView()
  {;
  }
  MyMapView(MyFrame * parent, wxWindowID id = wxID_ANY);
  virtual ~ MyMapView();

  void ResetScreenBitmap();
  void PrepareMap();
  void SetFullExtent();
  void SetBestZoom();
  void Invalidate();
  bool IsValidMap()
  {
    return ValidMap;
  }
  void DrawVectorLayer(wxGraphicsContext * gr, int layer);
  void DrawPoint(wxGraphicsContext * gr, int layer, gaiaGeomCollPtr geom);
  void DrawPolygon(wxGraphicsContext * gr, char color, gaiaGeomCollPtr geom);
  void DrawLinestring(wxGraphicsContext * gr, wxColour & colour,
                      gaiaGeomCollPtr geom, bool railways = false);
  void DrawRasterLayer(wxGraphicsContext * gr, int layer);
  void DrawDynamicGeometry(wxGraphicsContext * gr, int thick);
  void DrawDistanceMarker(wxGraphicsContext * gr, double x, double y);
  void ComputePlanarDistance(double *length);
  bool ComputePlanarLength(double *length);
  void DrawDistancePath(wxGraphicsContext * gr);
  void ComputeGreatCircleArc(double *great_circle, double *geodesic);
  bool ComputeGreatCircleLength(double *great_circle, double *geodesic);
  bool GetRaster(void *handle, wxBitmap * bmp);
  void DragMap(int x, int y);
  void OnTheFlyIdentify(int mouse_x, int mouse_y);
  void ShowMapTip(wxString & label, int x, int y, int category);
  void ClearMapTip();
  bool IsMapTipShown();
  void DoIdentify(int x, int y);
  void DoIdentifyCountry(int x, int y);
  void DoIdentifyAdminArea(int x, int y);
  bool LayerIsVisible(int layer);
  bool Graticules5Degrees();
  bool Graticules10Degrees();
  void ResetCurrentGeometry();
  void SetCurrentGeometry(gaiaGeomCollPtr geom);

  void OnSize(wxSizeEvent & event);
  void OnPaint(wxPaintEvent & event);
  void OnMouseMove(wxMouseEvent & event);
  void OnMouseWheel(wxMouseEvent & event);
  void OnTimerMouseWheel(wxTimerEvent & event);
  void OnTimerMouseMove(wxTimerEvent & event);
  void OnTimerDynamic(wxTimerEvent & event);
  void OnMouseClick(wxMouseEvent & event);
  void OnContextMenu(wxMouseEvent & event);
  void OnMouseDragStop(wxMouseEvent & event);
  void OnEraseBackground(wxEraseEvent & WXUNUSED(event))
  {;
  }
};

class CountryInfoDialog:public wxDialog
{
//
// a dialog to show Country infos
//
private:
  MyMapView * Parent;
  wxString Continent;
  wxString Country;
  wxString Capital;
  wxString Currency;
  wxString Population;
  wxString AreaKm2;
  wxString PopKm2;
  wxBitmap Flag;
  wxBitmap Emblem;
  wxTextCtrl *ContinentCtrl;
  wxTextCtrl *CountryCtrl;
  wxTextCtrl *CapitalCtrl;
  wxTextCtrl *CurrencyCtrl;
  wxTextCtrl *PopulationCtrl;
  wxTextCtrl *AreaKm2Ctrl;
  wxTextCtrl *PopKm2Ctrl;
  wxStaticBitmap *FlagCtrl;
  wxStaticBitmap *EmblemCtrl;
public:
    CountryInfoDialog(MyMapView * parent, wxString & continent_code,
                      wxString & continent_name, wxString & country_code,
                      wxString & country_name, wxString & capital,
                      wxString & currency_code, wxString & currency_name,
                      double area_km2, int population, double pop_km2,
                      wxBitmap & flag, wxBitmap & emblem);
  bool Create(MyFrame * parent);
    virtual ~ CountryInfoDialog()
  {;
  }
  void CreateControls();
  void OnOk(wxCommandEvent & event);
};

class AdminAreaInfoDialog:public wxDialog
{
//
// a dialog to show AdminArea infos
//
private:
  MyMapView * Parent;
  wxString Name;
  wxString Type;
  wxBitmap CoatOfArms;
  wxTextCtrl *NameCtrl;
  wxTextCtrl *TypeCtrl;
  wxStaticBitmap *CoatOfArmsCtrl;
public:
    AdminAreaInfoDialog(MyMapView * parent, wxString & name,
                        wxString & type, wxBitmap & coat_of_arms);
  bool Create(MyFrame * parent);
    virtual ~ AdminAreaInfoDialog()
  {;
  }
  void CreateControls();
  void OnOk(wxCommandEvent & event);
};

class MyFrame:public wxFrame
{
//
// the main GUI frame
//
private:
  wxString ConfigDbPath;        // PERSISTENCY - the last opend DB path
  sqlite3 *SqliteHandle;        // handle for SQLite DB
  wxString SqlitePath;          // path of SQLite DB
  MyMapView *MapView;           // the map panel
  wxString MapInfo;             // the current map description
  int CurrentMonth;             // the currently set month for TMean/Rain maps
  wxBitmap *BtnOpen;            // button icon for ATLAS OPEN
  wxBitmap *BtnClose;           // button icon for ATLAS CLOSE
  wxBitmap *BtnPolitical;       // button icon for Political map
  wxBitmap *BtnSatellite;       // button icon for Satellite map
  wxBitmap *BtnRelief;          // button icon for Relief map
  wxBitmap *BtnBathymetry;      // button icon for Bathymetry map
  wxBitmap *BtnTemperature;     // button icon for Temperature map
  wxBitmap *BtnRain;            // button icon for Rain map
  wxBitmap *BtnMonth;           // button icon for Month selection
  wxBitmap *BtnCenter;          // button icon for Zoom to Full Extent
  wxBitmap *BtnBestZoom;        // button icon for Zoom to Raster full resolution
  wxBitmap *BtnIdentify;        // buttom icon for Identify
  wxBitmap *BtnZoomIn;          // button icon for Zoom In
  wxBitmap *BtnZoomOut;         // buttom icon for Zoom Out
  wxBitmap *BtnPan;             // button icon for Pan
  wxBitmap *BtnGraticule;       // button icon for show/hide graticule
  wxBitmap *BtnDistance;        // button icon for Distance
  wxBitmap *BtnAbout;           // button icon for ABOUT
  wxBitmap *BtnExit;            // button icon for EXIT
  void *SatelliteHandle;        // handle to Satellite RasterLite datasource
  void *ReliefHandle;           // handle to Relief RasterLite datasource
  void *BathymetryHandle;       // handle to Bathymetry RasterLite datasource
  void *TempJanHandle;          // handle to TempJan RasterLite datasource
  void *TempAprHandle;          // handle to TempApr RasterLite datasource
  void *TempJulHandle;          // handle to TempJul RasterLite datasource
  void *TempOctHandle;          // handle to TempOct RasterLite datasource
  void *RainJanHandle;          // handle to RainJan RasterLite datasource
  void *RainAprHandle;          // handle to RainApr RasterLite datasource
  void *RainJulHandle;          // handle to RainJul RasterLite datasource
  void *RainOctHandle;          // handle to RainOct RasterLite datasource
  bool IsPolitical;             // current map is: Political
  bool IsSatellite;             // current map is: Satellite
  bool IsRelief;                // current map is: Relief
  bool IsBathymetry;            // current map is: Bathymetry
  bool IsTemperature;           // current map is: Temperature
  bool IsRain;                  // current map is: Rain
  bool IsIdentify;              // current map click is: Identify
  bool IsZoomIn;                // current map click is: ZoomIn
  bool IsZoomOut;               // current map click is: ZoomOut
  bool IsPan;                   // current map click is: Pan
  bool IsGraticule;             // show/hide graticule
  bool IsDistance;              // placing Distance markers
  void OpenSatellite();
  void OpenRelief();
  void OpenBathymetry();
  void OpenTempJan();
  void OpenTempApr();
  void OpenTempJul();
  void OpenTempOct();
  void OpenRainJan();
  void OpenRainApr();
  void OpenRainJul();
  void OpenRainOct();
  wxString SatellitePrefix;
  wxString ReliefPrefix;
  wxString BathymetryPrefix;
  wxString TempJanPrefix;
  wxString TempAprPrefix;
  wxString TempJulPrefix;
  wxString TempOctPrefix;
  wxString RainJanPrefix;
  wxString RainAprPrefix;
  wxString RainJulPrefix;
  wxString RainOctPrefix;
  wxString CountriesTable;
  wxString GeographicLinesTable;
  wxString GraticulesTable;
  wxString LakesTable;
  wxString LandAreasTable;
  wxString MarineAreasTable;
  wxString RiversTable;
  wxString TownsTable;
  wxString AdminAreasTable;
  wxString CulturalHeritageTable;
  wxString NamesTable;
  wxString HighwaysTable;
  wxString NaturalReservesTable;
  wxString RailwayStationsTable;
  wxString RailwaysTable;
  wxString TownAreasTable;
  int Srid;
  bool UseLongLatCoords;
  bool WorldWideMap;
  bool OkGraticule;
  bool IdentifyCountry;
  bool IdentifyAdminArea;
  double MapMinX;
  double MapMinY;
  double MapMaxX;
  double MapMaxY;
  bool CheckCountriesLayout(const char *name);
  bool CheckGeographicLinesLayout(const char *name);
  bool CheckGraticulesLayout(const char *name);
  bool CheckGenericWithClassLayout(const char *name);
  bool CheckGenericLayout(const char *name);
  bool CheckTownsLayout(const char *name);
  bool CheckAdminAreasLayout(const char *name);
  bool CheckRailwaysLayout(const char *name);
  bool SetCoordinatesLongLat();
  bool ComputeMapExtent();
public:
    MyFrame(const wxString & title, const wxPoint & pos, const wxSize & size);
    virtual ~ MyFrame();

  void SaveConfig();
  void LoadConfig();
  bool CheckDB();
  void UpdateStatusBar();
  sqlite3 *GetSqlite()
  {
    return SqliteHandle;
  }
  void *GetSatelliteHandle()
  {
    return SatelliteHandle;
  }
  void *GetReliefHandle()
  {
    return ReliefHandle;
  }
  void *GetBathymetryHandle()
  {
    return BathymetryHandle;
  }
  void *GetTempJanHandle()
  {
    return TempJanHandle;
  }
  void *GetTempAprHandle()
  {
    return TempAprHandle;
  }
  void *GetTempJulHandle()
  {
    return TempJulHandle;
  }
  void *GetTempOctHandle()
  {
    return TempOctHandle;
  }
  void *GetRainJanHandle()
  {
    return RainJanHandle;
  }
  void *GetRainAprHandle()
  {
    return RainAprHandle;
  }
  void *GetRainJulHandle()
  {
    return RainJulHandle;
  }
  void *GetRainOctHandle()
  {
    return RainOctHandle;
  }
  bool OpenDB();
  void CloseDB();
  int GetSrid()
  {
    return Srid;
  }
  bool IsPoliticalMap()
  {
    return IsPolitical;
  }
  bool IsSatelliteMap()
  {
    return IsSatellite;
  }
  bool IsReliefMap()
  {
    return IsRelief;
  }
  bool IsBathymetryMap()
  {
    return IsBathymetry;
  }
  bool IsTMeanJanMap();
  bool IsTMeanAprMap();
  bool IsTMeanJulMap();
  bool IsTMeanOctMap();
  bool IsRainJanMap();
  bool IsRainAprMap();
  bool IsRainJulMap();
  bool IsRainOctMap();
  bool IsModeIdentify()
  {
    return IsIdentify;
  }
  bool IsModeZoomIn()
  {
    return IsZoomIn;
  }
  bool IsModeZoomOut()
  {
    return IsZoomOut;
  }
  bool IsModePan()
  {
    return IsPan;
  }
  bool IsModeGraticule()
  {
    return IsGraticule;
  }
  bool IsModeDistance()
  {
    return IsDistance;
  }

  wxBitmap *GetBtnCenter()
  {
    return BtnCenter;
  }
  wxBitmap *GetBtnBestZoom()
  {
    return BtnBestZoom;
  }
  double GetMapMinX()
  {
    return MapMinX;
  }
  double GetMapMinY()
  {
    return MapMinY;
  }
  double GetMapMaxX()
  {
    return MapMaxX;
  }
  double GetMapMaxY()
  {
    return MapMaxY;
  }
  bool IsGeographic()
  {
    return UseLongLatCoords;
  }
  bool IsWorldWideMap()
  {
    return WorldWideMap;
  }

  bool HasCountries()
  {
    if (CountriesTable.Len() > 0)
      return true;
    else
      return false;
  }
  bool HasGeographicLines()
  {
    if (GeographicLinesTable.Len() > 0)
      return true;
    else
      return false;
  }
  bool HasGraticules()
  {
    if (GraticulesTable.Len() > 0)
      return true;
    else
      return false;
  }
  bool HasLakes()
  {
    if (LakesTable.Len() > 0)
      return true;
    else
      return false;
  }
  bool HasLandAreas()
  {
    if (LandAreasTable.Len() > 0)
      return true;
    else
      return false;
  }
  bool HasMarineAreas()
  {
    if (MarineAreasTable.Len() > 0)
      return true;
    else
      return false;
  }
  bool HasRivers()
  {
    if (RiversTable.Len() > 0)
      return true;
    else
      return false;
  }
  bool HasTowns()
  {
    if (TownsTable.Len() > 0)
      return true;
    else
      return false;
  }
  bool HasAdminAreas()
  {
    if (AdminAreasTable.Len() > 0)
      return true;
    else
      return false;
  }
  bool HasCulturalHeritage()
  {
    if (CulturalHeritageTable.Len() > 0)
      return true;
    else
      return false;
  }
  bool HasNames()
  {
    if (NamesTable.Len() > 0)
      return true;
    else
      return false;
  }
  bool HasHighways()
  {
    if (HighwaysTable.Len() > 0)
      return true;
    else
      return false;
  }
  bool HasNaturalReserves()
  {
    if (NaturalReservesTable.Len() > 0)
      return true;
    else
      return false;
  }
  bool HasRailwayStations()
  {
    if (RailwayStationsTable.Len() > 0)
      return true;
    else
      return false;
  }
  bool HasRailways()
  {
    if (RailwaysTable.Len() > 0)
      return true;
    else
      return false;
  }
  bool HasTownAreas()
  {
    if (TownAreasTable.Len() > 0)
      return true;
    else
      return false;
  }
  bool HasGraticule()
  {
    return OkGraticule;
  }
  bool HasIdentify()
  {
    if (HasCountries() || HasAdminAreas())
      return true;
    else
      return false;
  }
  bool IsIdentifyCountry()
  {
    return IdentifyCountry;
  }
  bool IsIdentifyAdminArea()
  {
    return IdentifyAdminArea;
  }
  wxString & GetCountries()
  {
    return CountriesTable;
  }
  wxString & GetGeographicLines()
  {
    return GeographicLinesTable;
  }
  wxString & GetGraticules()
  {
    return GraticulesTable;
  }
  wxString & GetLakes()
  {
    return LakesTable;
  }
  wxString & GetLandAreas()
  {
    return LandAreasTable;
  }
  wxString & GetMarineAreas()
  {
    return MarineAreasTable;
  }
  wxString & GetRivers()
  {
    return RiversTable;
  }
  wxString & GetTowns()
  {
    return TownsTable;
  }
  wxString & GetAdminAreas()
  {
    return AdminAreasTable;
  }
  wxString & GetCulturalHeritage()
  {
    return CulturalHeritageTable;
  }
  wxString & GetNames()
  {
    return NamesTable;
  }
  wxString & GetHighways()
  {
    return HighwaysTable;
  }
  wxString & GetNaturalReserves()
  {
    return NaturalReservesTable;
  }
  wxString & GetRailwayStations()
  {
    return RailwayStationsTable;
  }
  wxString & GetRailways()
  {
    return RailwaysTable;
  }
  wxString & GetTownAreas()
  {
    return TownAreasTable;
  }

  void UpdateCoords(wxString & coords);
  void SetMapInfo(int type);
  void UpdateMapInfo();
  double GetRasterMaxResolution(double current);
  void GetSRSName(wxString & srs);

  void OnOpen(wxCommandEvent & event);
  void OnClose(wxCommandEvent & event);
  void OnPolitical(wxCommandEvent & event);
  void OnSatellite(wxCommandEvent & event);
  void OnRelief(wxCommandEvent & event);
  void OnBathymetry(wxCommandEvent & event);
  void OnTemperature(wxCommandEvent & event);
  void OnRain(wxCommandEvent & event);
  void OnMonth(wxCommandEvent & event);
  void OnJanuary(wxCommandEvent & event);
  void OnApril(wxCommandEvent & event);
  void OnJuly(wxCommandEvent & event);
  void OnOctober(wxCommandEvent & event);
  void OnCenter(wxCommandEvent & event);
  void OnBestZoom(wxCommandEvent & event);
  void OnIdentify(wxCommandEvent & event);
  void OnZoomIn(wxCommandEvent & event);
  void OnZoomOut(wxCommandEvent & event);
  void OnPan(wxCommandEvent & event);
  void OnGraticule(wxCommandEvent & event);
  void OnDistance(wxCommandEvent & event);
  void OnQuit(wxCommandEvent & event);
  void OnAbout(wxCommandEvent & event);
  void OnSize(wxSizeEvent & event);
};
