/*
/ Main.cpp
/ the main core of LibreAtlas
/
/ version 1.0, 2012 May 16
/
/ Author: Sandro Furieri a-furieri@lqt.it
/
/ Copyright (C) 2012  Alessandro Furieri
/
/    This program is free software: you can redistribute it and/or modify
/    it under the terms of the GNU General Public License as published by
/    the Free Software Foundation, either version 3 of the License, or
/    (at your option) any later version.
/
/    This program is distributed in the hope that it will be useful,
/    but WITHOUT ANY WARRANTY; without even the implied warranty of
/    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
/    GNU General Public License for more details.
/
/    You should have received a copy of the GNU General Public License
/    along with this program.  If not, see <http://www.gnu.org/licenses/>.
/
*/

#include "Classdef.h"

#include <float.h>

#include "wx/menu.h"
#include "wx/aboutdlg.h"
#include "wx/config.h"

#include <proj_api.h>
#include <geos_c.h>
//
// ICONs in XPM format [universally portable]
//
#include "icons/icon.xpm"
#include "icons/icon_info.xpm"
#include "icons/open.xpm"
#include "icons/close.xpm"
#include "icons/political.xpm"
#include "icons/phys_marble.xpm"
#include "icons/phys_srtm30.xpm"
#include "icons/phys_noaa.xpm"
#include "icons/temperature.xpm"
#include "icons/rain.xpm"
#include "icons/month.xpm"
#include "icons/center.xpm"
#include "icons/best_zoom.xpm"
#include "icons/identify.xpm"
#include "icons/zoom_in.xpm"
#include "icons/zoom_out.xpm"
#include "icons/pan.xpm"
#include "icons/graticule.xpm"
#include "icons/pin.xpm"
#include "icons/about.xpm"
#include "icons/exit.xpm"

IMPLEMENT_APP(MyApp)
     bool MyApp::OnInit()
{
//
// main APP implementation
//
  MyFrame *frame = new MyFrame(wxT("LibreAtlas      [free Atlas viewer]"),
                               wxPoint(0, 0), wxSize(800, 600));
  frame->Show(true);
  SetTopWindow(frame);
  frame->LoadConfig();
  return true;
}

MyFrame::MyFrame(const wxString & title, const wxPoint & pos, const wxSize & size):
wxFrame((wxFrame *) NULL, -1, title, pos,
        size)
{
//
// main GUI frame constructor
//

// forcing DECIMAL POINT IS COMMA
  setlocale(LC_NUMERIC, "C");

  SqliteHandle = NULL;
  SqlitePath = wxT("");
  MapInfo = wxT("");
  SatelliteHandle = NULL;
  ReliefHandle = NULL;
  BathymetryHandle = NULL;
  TempJanHandle = NULL;
  TempAprHandle = NULL;
  TempJulHandle = NULL;
  TempOctHandle = NULL;
  RainJanHandle = NULL;
  RainAprHandle = NULL;
  RainJulHandle = NULL;
  RainOctHandle = NULL;
  IsPolitical = false;
  IsSatellite = false;
  IsRelief = false;
  IsBathymetry = false;
  IsTemperature = false;
  IsRain = false;
  IsZoomIn = true;
  IsZoomOut = false;
  IsPan = false;
  IsGraticule = false;
  IsIdentify = false;
  IsDistance = false;
  IsZoomIn = true;
  IsZoomOut = false;
  IsPan = false;
  CurrentMonth = 1;
  MapView = new MyMapView(this);

  spatialite_init(0);           // loading the SpatiaLite extension

  ::wxInitAllImageHandlers();

  BtnOpen = new wxBitmap(open_xpm);
  BtnClose = new wxBitmap(close_xpm);
  BtnPolitical = new wxBitmap(political_xpm);
  BtnSatellite = new wxBitmap(phys_marble_xpm);
  BtnRelief = new wxBitmap(phys_srtm30_xpm);
  BtnBathymetry = new wxBitmap(phys_noaa_xpm);
  BtnTemperature = new wxBitmap(temperature_xpm);
  BtnRain = new wxBitmap(rain_xpm);
  BtnMonth = new wxBitmap(month_xpm);
  BtnCenter = new wxBitmap(center_xpm);
  BtnBestZoom = new wxBitmap(best_zoom_xpm);
  BtnIdentify = new wxBitmap(identify_xpm);
  BtnZoomIn = new wxBitmap(zoom_in_xpm);
  BtnZoomOut = new wxBitmap(zoom_out_xpm);
  BtnPan = new wxBitmap(pan_xpm);
  BtnGraticule = new wxBitmap(graticule_xpm);
  BtnDistance = new wxBitmap(pin_xpm);
  BtnAbout = new wxBitmap(about_xpm);
  BtnExit = new wxBitmap(exit_xpm);

//
// setting up the application icon
//      
  wxIcon MyIcon(icon_xpm);
  SetIcon(MyIcon);

//
// setting up the status bar
//
  wxStatusBar *statusBar = new wxStatusBar(this);
  statusBar->SetFieldsCount(3);
  int fldWidth[3];
  fldWidth[0] = -3;
  fldWidth[1] = -1;
  fldWidth[2] = -2;
  statusBar->SetStatusWidths(3, fldWidth);
  SetStatusBar(statusBar);

//
// setting up the menu bar
//
  wxMenu *menuFile = new wxMenu;
  wxMenuItem *menuItem;
  menuItem = new wxMenuItem(menuFile, wxID_OPEN, wxT("&Open an Atlas"));
  menuItem->SetBitmap(*BtnOpen);
  menuFile->Append(menuItem);
  menuItem = new wxMenuItem(menuFile, wxID_CLOSE, wxT("&Close the Atlas"));
  menuItem->SetBitmap(*BtnClose);
  menuFile->Append(menuItem);
  menuFile->AppendSeparator();
  menuItem =
    new wxMenuItem(menuFile, ID_Political,
                   wxT("&Political Map"), wxT(""), wxITEM_RADIO);
  menuFile->Append(menuItem);
  menuItem =
    new wxMenuItem(menuFile, ID_Satellite,
                   wxT("&Satellite Map"), wxT(""), wxITEM_RADIO);
  menuFile->Append(menuItem);
  menuItem =
    new wxMenuItem(menuFile, ID_Relief,
                   wxT("&Relief Map"), wxT(""), wxITEM_RADIO);
  menuFile->Append(menuItem);
  menuItem =
    new wxMenuItem(menuFile, ID_Bathymetry,
                   wxT("&Bathymetry Map"), wxT(""), wxITEM_RADIO);
  menuFile->Append(menuItem);
  menuItem =
    new wxMenuItem(menuFile, ID_Temperature,
                   wxT("Mean &Temperature"), wxT(""), wxITEM_RADIO);
  menuFile->Append(menuItem);
  menuItem =
    new wxMenuItem(menuFile, ID_Rain,
                   wxT("&Precipitation (rain)"), wxT(""), wxITEM_RADIO);
  menuFile->Append(menuItem);
  menuFile->AppendSeparator();
  menuItem =
    new wxMenuItem(menuFile, ID_January,
                   wxT("current month: January"), wxT(""), wxITEM_RADIO);
  menuFile->Append(menuItem);
  menuItem =
    new wxMenuItem(menuFile, ID_April,
                   wxT("current month: April"), wxT(""), wxITEM_RADIO);
  menuFile->Append(menuItem);
  menuItem =
    new wxMenuItem(menuFile, ID_July,
                   wxT("current month: July"), wxT(""), wxITEM_RADIO);
  menuFile->Append(menuItem);
  menuItem =
    new wxMenuItem(menuFile, ID_October,
                   wxT("current month: October"), wxT(""), wxITEM_RADIO);
  menuFile->Append(menuItem);
  menuFile->AppendSeparator();
  menuItem =
    new wxMenuItem(menuFile, ID_Graticule,
                   wxT("Graticule"), wxT(""), wxITEM_CHECK);
  menuFile->Append(menuItem);
  menuFile->AppendSeparator();
  menuItem = new wxMenuItem(menuFile, ID_Center, wxT("Zoom to &Full Extent"));
  menuItem->SetBitmap(*BtnCenter);
  menuFile->Append(menuItem);
  menuItem =
    new wxMenuItem(menuFile, ID_BestZoom,
                   wxT("Zoom to &Raster full resolution"));
  menuItem->SetBitmap(*BtnBestZoom);
  menuFile->Append(menuItem);
  menuItem =
    new wxMenuItem(menuFile, ID_ZoomIn, wxT("Zoom &In"), wxT(""), wxITEM_RADIO);
  menuFile->Append(menuItem);
  menuItem =
    new wxMenuItem(menuFile, ID_ZoomOut,
                   wxT("Zoom &Out"), wxT(""), wxITEM_RADIO);
  menuFile->Append(menuItem);
  menuItem =
    new wxMenuItem(menuFile, ID_Pan, wxT("&Pan"), wxT(""), wxITEM_RADIO);
  menuFile->Append(menuItem);
  menuItem =
    new wxMenuItem(menuFile, ID_Identify,
                   wxT("Identify"), wxT(""), wxITEM_RADIO);
  menuFile->Append(menuItem);
  menuItem =
    new wxMenuItem(menuFile, ID_Distance,
                   wxT("Measure Distance"), wxT(""), wxITEM_RADIO);
  menuFile->Append(menuItem);
  menuFile->AppendSeparator();
  menuItem = new wxMenuItem(menuFile, wxID_ABOUT, wxT("&About ..."));
  menuItem->SetBitmap(*BtnAbout);
  menuFile->Append(menuItem);
  menuFile->AppendSeparator();
  menuItem = new wxMenuItem(menuFile, wxID_EXIT, wxT("&Quit"));
  menuItem->SetBitmap(*BtnExit);
  menuFile->Append(menuItem);
  wxMenuBar *menuBar = new wxMenuBar;
  menuBar->Append(menuFile, wxT("&Files"));
  SetMenuBar(menuBar);

//
// setting up menu initial state 
//
  menuBar->Enable(wxID_CLOSE, false);
  menuBar->Enable(ID_Political, false);
  menuBar->Enable(ID_Satellite, false);
  menuBar->Enable(ID_Relief, false);
  menuBar->Enable(ID_Bathymetry, false);
  menuBar->Enable(ID_Temperature, false);
  menuBar->Enable(ID_Rain, false);
  menuBar->Enable(ID_January, false);
  menuBar->Enable(ID_April, false);
  menuBar->Enable(ID_July, false);
  menuBar->Enable(ID_October, false);
  menuBar->Enable(ID_Center, false);
  menuBar->Enable(ID_BestZoom, false);
  menuBar->Enable(ID_Identify, false);
  menuBar->Enable(ID_ZoomIn, false);
  menuBar->Enable(ID_ZoomOut, false);
  menuBar->Enable(ID_Pan, false);
  menuBar->Enable(ID_Graticule, false);
  menuBar->Enable(ID_Distance, false);

//
// setting up the toolbar
//      
  wxToolBar *toolBar = CreateToolBar();
  toolBar->AddTool(wxID_OPEN, wxT("Open an Atlas"),
                   *BtnOpen, wxNullBitmap, wxITEM_NORMAL, wxT("Open an Atlas"));
  toolBar->AddTool(wxID_CLOSE, wxT("Close the Atlas"),
                   *BtnClose, wxNullBitmap, wxITEM_NORMAL,
                   wxT("Close the Atlas"));
  toolBar->AddSeparator();
  toolBar->AddTool(ID_Political, wxT("Political Map"),
                   *BtnPolitical, wxNullBitmap, wxITEM_RADIO,
                   wxT("Political Map"));
  toolBar->AddTool(ID_Satellite, wxT("Satellite Map"),
                   *BtnSatellite, wxNullBitmap, wxITEM_RADIO,
                   wxT("Satellite Map"));
  toolBar->AddTool(ID_Relief, wxT("Relief Map"),
                   *BtnRelief, wxNullBitmap, wxITEM_RADIO, wxT("Relief Map"));
  toolBar->AddTool(ID_Bathymetry, wxT("Bathymetry Map"),
                   *BtnBathymetry, wxNullBitmap, wxITEM_RADIO,
                   wxT("Bathymetry Map"));
  toolBar->AddTool(ID_Temperature, wxT("mean Temperature Map"), *BtnTemperature,
                   wxNullBitmap, wxITEM_RADIO, wxT("mean Temperature Map"));
  toolBar->AddTool(ID_Rain, wxT("Precipitation (rain) Map"), *BtnRain,
                   wxNullBitmap, wxITEM_RADIO, wxT("Precipitation (rain) Map"));
  toolBar->AddTool(ID_Month, wxT("Change Month"), *BtnMonth, wxNullBitmap,
                   wxITEM_NORMAL, wxT("Change Month"));
  toolBar->AddSeparator();
  toolBar->AddTool(ID_Graticule, wxT("Graticule"),
                   *BtnGraticule, wxNullBitmap, wxITEM_CHECK, wxT("Graticule"));
  toolBar->AddSeparator();
  toolBar->AddTool(ID_Center, wxT("Zoom to Full Extent"),
                   *BtnCenter, wxNullBitmap, wxITEM_NORMAL,
                   wxT("Zoom to Full Extent"));
  toolBar->AddTool(ID_BestZoom, wxT("Zoom to Raster full resolution"),
                   *BtnBestZoom, wxNullBitmap, wxITEM_NORMAL,
                   wxT("Zoom to Raster full resolution"));
  toolBar->AddTool(ID_ZoomIn, wxT("Zoom In"), *BtnZoomIn, wxNullBitmap,
                   wxITEM_RADIO, wxT("Zoom In"));
  toolBar->AddTool(ID_ZoomOut, wxT("Zoom Out"), *BtnZoomOut, wxNullBitmap,
                   wxITEM_RADIO, wxT("Zoom Out"));
  toolBar->AddTool(ID_Pan, wxT("Pan"), *BtnPan, wxNullBitmap, wxITEM_RADIO,
                   wxT("Pan"));
  toolBar->AddTool(ID_Identify, wxT("Identify"),
                   *BtnIdentify, wxNullBitmap, wxITEM_RADIO, wxT("Identify"));
  toolBar->AddTool(ID_Distance, wxT("Measure Distance"),
                   *BtnDistance, wxNullBitmap, wxITEM_RADIO,
                   wxT("Measure Distance"));
  toolBar->AddSeparator();
  toolBar->AddTool(wxID_ABOUT, wxT("About ..."), *BtnAbout, wxNullBitmap,
                   wxITEM_NORMAL, wxT("About ..."));
  toolBar->AddSeparator();
  toolBar->AddTool(wxID_EXIT, wxT("Quit"), *BtnExit, wxNullBitmap,
                   wxITEM_NORMAL, wxT("Quit"));
  toolBar->Realize();
  SetToolBar(toolBar);

//
// setting up the toolbar initial state
//
  toolBar->EnableTool(wxID_CLOSE, false);
  toolBar->EnableTool(ID_Political, false);
  toolBar->EnableTool(ID_Satellite, false);
  toolBar->EnableTool(ID_Relief, false);
  toolBar->EnableTool(ID_Bathymetry, false);
  toolBar->EnableTool(ID_Temperature, false);
  toolBar->EnableTool(ID_Rain, false);
  toolBar->EnableTool(ID_Month, false);
  toolBar->EnableTool(ID_Center, false);
  toolBar->EnableTool(ID_Identify, false);
  toolBar->EnableTool(ID_ZoomIn, false);
  toolBar->EnableTool(ID_ZoomOut, false);
  toolBar->EnableTool(ID_Pan, false);
  toolBar->EnableTool(ID_Graticule, false);
  toolBar->EnableTool(ID_Distance, false);

//
// setting up event handlers for menu and toolbar
//
  Connect(wxID_OPEN, wxEVT_COMMAND_MENU_SELECTED,
          (wxObjectEventFunction) & MyFrame::OnOpen);
  Connect(wxID_CLOSE, wxEVT_COMMAND_MENU_SELECTED,
          (wxObjectEventFunction) & MyFrame::OnClose);
  Connect(ID_Political, wxEVT_COMMAND_MENU_SELECTED,
          (wxObjectEventFunction) & MyFrame::OnPolitical);
  Connect(ID_Satellite, wxEVT_COMMAND_MENU_SELECTED,
          (wxObjectEventFunction) & MyFrame::OnSatellite);
  Connect(ID_Relief, wxEVT_COMMAND_MENU_SELECTED,
          (wxObjectEventFunction) & MyFrame::OnRelief);
  Connect(ID_Bathymetry, wxEVT_COMMAND_MENU_SELECTED,
          (wxObjectEventFunction) & MyFrame::OnBathymetry);
  Connect(ID_Temperature, wxEVT_COMMAND_MENU_SELECTED,
          (wxObjectEventFunction) & MyFrame::OnTemperature);
  Connect(ID_Rain, wxEVT_COMMAND_MENU_SELECTED,
          (wxObjectEventFunction) & MyFrame::OnRain);
  Connect(ID_Month, wxEVT_COMMAND_MENU_SELECTED,
          (wxObjectEventFunction) & MyFrame::OnMonth);
  Connect(ID_January, wxEVT_COMMAND_MENU_SELECTED,
          (wxObjectEventFunction) & MyFrame::OnJanuary);
  Connect(ID_April, wxEVT_COMMAND_MENU_SELECTED,
          (wxObjectEventFunction) & MyFrame::OnApril);
  Connect(ID_July, wxEVT_COMMAND_MENU_SELECTED,
          (wxObjectEventFunction) & MyFrame::OnJuly);
  Connect(ID_October, wxEVT_COMMAND_MENU_SELECTED,
          (wxObjectEventFunction) & MyFrame::OnOctober);
  Connect(ID_Center, wxEVT_COMMAND_MENU_SELECTED,
          (wxObjectEventFunction) & MyFrame::OnCenter);
  Connect(ID_BestZoom, wxEVT_COMMAND_MENU_SELECTED,
          (wxObjectEventFunction) & MyFrame::OnBestZoom);
  Connect(ID_Identify, wxEVT_COMMAND_MENU_SELECTED,
          (wxObjectEventFunction) & MyFrame::OnIdentify);
  Connect(ID_Distance, wxEVT_COMMAND_MENU_SELECTED,
          (wxObjectEventFunction) & MyFrame::OnDistance);
  Connect(ID_ZoomIn, wxEVT_COMMAND_MENU_SELECTED,
          (wxObjectEventFunction) & MyFrame::OnZoomIn);
  Connect(ID_ZoomOut, wxEVT_COMMAND_MENU_SELECTED,
          (wxObjectEventFunction) & MyFrame::OnZoomOut);
  Connect(ID_Pan, wxEVT_COMMAND_MENU_SELECTED,
          (wxObjectEventFunction) & MyFrame::OnPan);
  Connect(ID_Graticule, wxEVT_COMMAND_MENU_SELECTED,
          (wxObjectEventFunction) & MyFrame::OnGraticule);
  Connect(wxID_ABOUT, wxEVT_COMMAND_MENU_SELECTED,
          (wxObjectEventFunction) & MyFrame::OnAbout);
  Connect(wxID_EXIT, wxEVT_COMMAND_MENU_SELECTED,
          (wxObjectEventFunction) & MyFrame::OnQuit);
  Connect(wxID_ANY, wxEVT_SIZE, (wxObjectEventFunction) & MyFrame::OnSize);
}

MyFrame::~MyFrame()
{
//
// main GUI frame destructor
//
  SaveConfig();
  if (SqliteHandle)
    sqlite3_close(SqliteHandle);
  if (SatelliteHandle)
    rasterliteClose(SatelliteHandle);
  if (ReliefHandle)
    rasterliteClose(ReliefHandle);
  if (BathymetryHandle)
    rasterliteClose(BathymetryHandle);
  if (TempJanHandle)
    rasterliteClose(TempJanHandle);
  if (TempAprHandle)
    rasterliteClose(TempAprHandle);
  if (TempJulHandle)
    rasterliteClose(TempJulHandle);
  if (TempOctHandle)
    rasterliteClose(TempOctHandle);
  if (RainJanHandle)
    rasterliteClose(RainJanHandle);
  if (RainAprHandle)
    rasterliteClose(RainAprHandle);
  if (RainJulHandle)
    rasterliteClose(RainJulHandle);
  if (RainOctHandle)
    rasterliteClose(RainOctHandle);
  if (BtnOpen != NULL)
    delete BtnOpen;
  if (BtnClose != NULL)
    delete BtnClose;
  if (BtnPolitical != NULL)
    delete BtnPolitical;
  if (BtnSatellite != NULL)
    delete BtnSatellite;
  if (BtnRelief != NULL)
    delete BtnRelief;
  if (BtnBathymetry != NULL)
    delete BtnBathymetry;
  if (BtnTemperature != NULL)
    delete BtnTemperature;
  if (BtnRain != NULL)
    delete BtnRain;
  if (BtnMonth != NULL)
    delete BtnMonth;
  if (BtnCenter != NULL)
    delete BtnCenter;
  if (BtnBestZoom != NULL)
    delete BtnBestZoom;
  if (BtnIdentify != NULL)
    delete BtnIdentify;
  if (BtnDistance != NULL)
    delete BtnDistance;
  if (BtnZoomIn != NULL)
    delete BtnZoomIn;
  if (BtnZoomOut != NULL)
    delete BtnZoomOut;
  if (BtnPan != NULL)
    delete BtnPan;
  if (BtnGraticule != NULL)
    delete BtnGraticule;
  if (BtnAbout != NULL)
    delete BtnAbout;
  if (BtnExit != NULL)
    delete BtnExit;
}

void MyFrame::OnSize(wxSizeEvent & WXUNUSED(event))
{
//
// this window has changed its size
//
  if (GetStatusBar() != NULL)
    {
      wxSize sz = GetClientSize();
      MapView->SetSize(0, 0, sz.GetWidth(), sz.GetHeight());
    }
}

void MyFrame::OnPolitical(wxCommandEvent & WXUNUSED(event))
{
//
// current map is: Political
//
  wxMenuBar *menuBar = GetMenuBar();
  wxToolBar *toolBar = GetToolBar();
  if (IsPolitical == false)
    {
      if (menuBar)
        {
          menuBar->Check(ID_Political, true);
          menuBar->Enable(ID_January, false);
          menuBar->Enable(ID_April, false);
          menuBar->Enable(ID_July, false);
          menuBar->Enable(ID_October, false);
          menuBar->Enable(ID_BestZoom, false);
        }
      if (toolBar)
        {
          toolBar->ToggleTool(ID_Political, true);
          toolBar->EnableTool(ID_Month, false);
          toolBar->EnableTool(ID_BestZoom, false);
        }
      IsPolitical = true;
      IsSatellite = false;
      IsRelief = false;
      IsBathymetry = false;
      IsTemperature = false;
      IsRain = false;
      MapView->PrepareMap();
    }
}

void MyFrame::OnSatellite(wxCommandEvent & WXUNUSED(event))
{
//
// current map is: Satellite
//
  wxMenuBar *menuBar = GetMenuBar();
  wxToolBar *toolBar = GetToolBar();
  if (IsSatellite == false)
    {
      if (menuBar)
        {
          menuBar->Check(ID_Satellite, true);
          menuBar->Enable(ID_January, false);
          menuBar->Enable(ID_April, false);
          menuBar->Enable(ID_July, false);
          menuBar->Enable(ID_October, false);
          menuBar->Enable(ID_BestZoom, true);
        }
      if (toolBar)
        {
          toolBar->ToggleTool(ID_Satellite, true);
          toolBar->EnableTool(ID_Month, false);
          toolBar->EnableTool(ID_BestZoom, true);
        }
      IsPolitical = false;
      IsSatellite = true;
      IsRelief = false;
      IsBathymetry = false;
      IsTemperature = false;
      IsRain = false;
      MapView->PrepareMap();
    }
}

void MyFrame::OnRelief(wxCommandEvent & WXUNUSED(event))
{
//
// current map is: Relief
//
  wxMenuBar *menuBar = GetMenuBar();
  wxToolBar *toolBar = GetToolBar();
  if (IsRelief == false)
    {
      if (menuBar)
        {
          menuBar->Check(ID_Relief, true);
          menuBar->Enable(ID_January, false);
          menuBar->Enable(ID_April, false);
          menuBar->Enable(ID_July, false);
          menuBar->Enable(ID_October, false);
          menuBar->Enable(ID_BestZoom, true);
        }
      if (toolBar)
        {
          toolBar->ToggleTool(ID_Relief, true);
          toolBar->EnableTool(ID_Month, false);
          toolBar->EnableTool(ID_BestZoom, true);
        }
      IsPolitical = false;
      IsSatellite = false;
      IsRelief = true;
      IsBathymetry = false;
      IsTemperature = false;
      IsRain = false;
      MapView->PrepareMap();
    }
}

void MyFrame::OnBathymetry(wxCommandEvent & WXUNUSED(event))
{
//
// current map is: Bathymetry
//
  wxMenuBar *menuBar = GetMenuBar();
  wxToolBar *toolBar = GetToolBar();
  if (IsBathymetry == false)
    {
      if (menuBar)
        {
          menuBar->Check(ID_Bathymetry, true);
          menuBar->Enable(ID_January, false);
          menuBar->Enable(ID_April, false);
          menuBar->Enable(ID_July, false);
          menuBar->Enable(ID_October, false);
          menuBar->Enable(ID_BestZoom, true);
        }
      if (toolBar)
        {
          toolBar->ToggleTool(ID_Bathymetry, true);
          toolBar->EnableTool(ID_Month, false);
          toolBar->EnableTool(ID_BestZoom, true);
        }
      IsPolitical = false;
      IsSatellite = false;
      IsRelief = false;
      IsBathymetry = true;
      IsTemperature = false;
      IsRain = false;
      MapView->PrepareMap();
    }
}

void MyFrame::OnTemperature(wxCommandEvent & WXUNUSED(event))
{
//
// current map is: Mean Temperature
//
  wxMenuBar *menuBar = GetMenuBar();
  wxToolBar *toolBar = GetToolBar();
  if (IsTemperature == false)
    {
      if (menuBar)
        {
          menuBar->Check(ID_Temperature, true);
          menuBar->Enable(ID_January, true);
          menuBar->Enable(ID_April, true);
          menuBar->Enable(ID_July, true);
          menuBar->Enable(ID_October, true);
          menuBar->Enable(ID_BestZoom, true);
          switch (CurrentMonth)
            {
              case 1:
                menuBar->Check(ID_January, true);
                break;
              case 4:
                menuBar->Check(ID_April, true);
                break;
              case 7:
                menuBar->Check(ID_July, true);
                break;
              default:
                menuBar->Check(ID_October, true);
                break;
            };
        }
      if (toolBar)
        {
          toolBar->ToggleTool(ID_Temperature, true);
          toolBar->EnableTool(ID_Month, true);
          toolBar->EnableTool(ID_BestZoom, true);
        }
      IsPolitical = false;
      IsSatellite = false;
      IsRelief = false;
      IsBathymetry = false;
      IsTemperature = true;
      IsRain = false;
      MapView->PrepareMap();
    }
}

void MyFrame::OnRain(wxCommandEvent & WXUNUSED(event))
{
//
// current map is: Precipitation (rain)
//
  wxMenuBar *menuBar = GetMenuBar();
  wxToolBar *toolBar = GetToolBar();
  if (IsRain == false)
    {
      if (menuBar)
        {
          menuBar->Check(ID_Rain, true);
          menuBar->Enable(ID_January, true);
          menuBar->Enable(ID_April, true);
          menuBar->Enable(ID_July, true);
          menuBar->Enable(ID_October, true);
          menuBar->Enable(ID_BestZoom, true);
          switch (CurrentMonth)
            {
              case 1:
                menuBar->Check(ID_January, true);
                break;
              case 4:
                menuBar->Check(ID_April, true);
                break;
              case 7:
                menuBar->Check(ID_July, true);
                break;
              default:
                menuBar->Check(ID_October, true);
                break;
            };
        }
      if (toolBar)
        {
          toolBar->ToggleTool(ID_Rain, true);
          toolBar->EnableTool(ID_Month, true);
          toolBar->EnableTool(ID_BestZoom, true);
        }
      IsPolitical = false;
      IsSatellite = false;
      IsRelief = false;
      IsBathymetry = false;
      IsTemperature = false;
      IsRain = true;
      MapView->PrepareMap();
    }
}

void MyFrame::OnMonth(wxCommandEvent & WXUNUSED(event))
{
//
// changing the current month
//
  wxMenuBar *menuBar = GetMenuBar();
  switch (CurrentMonth)
    {
      case 1:
        CurrentMonth = 4;
        menuBar->Check(ID_April, true);
        break;
      case 4:
        CurrentMonth = 7;
        menuBar->Check(ID_July, true);
        break;
      case 7:
        CurrentMonth = 10;
        menuBar->Check(ID_October, true);
        break;
      default:
        CurrentMonth = 1;
        menuBar->Check(ID_January, true);
        break;
    };
  MapView->PrepareMap();
}

void MyFrame::OnJanuary(wxCommandEvent & WXUNUSED(event))
{
//
// changing the current month: January
//
  if (CurrentMonth == 1)
    return;
  CurrentMonth = 1;
  GetMenuBar()->Check(ID_January, true);
  MapView->PrepareMap();
}

void MyFrame::OnApril(wxCommandEvent & WXUNUSED(event))
{
//
// changing the current month: April
//
  if (CurrentMonth == 4)
    return;
  CurrentMonth = 4;
  GetMenuBar()->Check(ID_April, true);
  MapView->PrepareMap();
}

void MyFrame::OnJuly(wxCommandEvent & WXUNUSED(event))
{
//
// changing the current month: July
//
  if (CurrentMonth == 7)
    return;
  CurrentMonth = 7;
  GetMenuBar()->Check(ID_July, true);
  MapView->PrepareMap();
}

void MyFrame::OnOctober(wxCommandEvent & WXUNUSED(event))
{
//
// changing the current month: October
//
  if (CurrentMonth == 10)
    return;
  CurrentMonth = 10;
  GetMenuBar()->Check(ID_October, true);
  MapView->PrepareMap();
}

void MyFrame::OnCenter(wxCommandEvent & WXUNUSED(event))
{
//
// resetting to Full Extent
//
  MapView->SetFullExtent();
  MapView->PrepareMap();
}

void MyFrame::OnBestZoom(wxCommandEvent & WXUNUSED(event))
{
//
// setting to Raster full resoluzion Zoom level
//
  MapView->SetBestZoom();
  MapView->PrepareMap();
}

void MyFrame::OnGraticule(wxCommandEvent & WXUNUSED(event))
{
//
// show/hide Graticule
//
  if (IsGraticule == true)
    IsGraticule = false;
  else
    IsGraticule = true;
  wxMenuBar *menuBar = GetMenuBar();
  wxToolBar *toolBar = GetToolBar();
  if (IsGraticule == true)
    {
      menuBar->Check(ID_Graticule, true);
      toolBar->ToggleTool(ID_Graticule, true);
  } else
    {
      menuBar->Check(ID_Graticule, false);
      toolBar->ToggleTool(ID_Graticule, false);
    }
  MapView->PrepareMap();
}

void MyFrame::OnIdentify(wxCommandEvent & WXUNUSED(event))
{
//
// current map click is: Identify
//
  wxMenuBar *menuBar = GetMenuBar();
  wxToolBar *toolBar = GetToolBar();
  if (IsIdentify == false)
    {
      if (menuBar)
        menuBar->Check(ID_Identify, true);
      if (toolBar)
        toolBar->ToggleTool(ID_Identify, true);
      IsIdentify = true;
      IsDistance = false;
      IsZoomIn = false;
      IsZoomOut = false;
      IsPan = false;
    }
}

void MyFrame::OnDistance(wxCommandEvent & WXUNUSED(event))
{
//
// current map click is: Distance
//
  wxMenuBar *menuBar = GetMenuBar();
  wxToolBar *toolBar = GetToolBar();
  if (IsDistance == false)
    {
      if (menuBar)
        menuBar->Check(ID_Distance, true);
      if (toolBar)
        toolBar->ToggleTool(ID_Distance, true);
      IsIdentify = false;
      IsDistance = true;
      IsZoomIn = false;
      IsZoomOut = false;
      IsPan = false;
    }
}

void MyFrame::OnZoomIn(wxCommandEvent & WXUNUSED(event))
{
//
// current map click is: ZoomIn
//
  wxMenuBar *menuBar = GetMenuBar();
  wxToolBar *toolBar = GetToolBar();
  if (IsZoomIn == false)
    {
      menuBar->Check(ID_ZoomIn, true);
      if (toolBar)
        toolBar->ToggleTool(ID_ZoomIn, true);
      IsIdentify = false;
      IsDistance = false;
      IsZoomIn = true;
      IsZoomOut = false;
      IsPan = false;
    }
}

void MyFrame::OnZoomOut(wxCommandEvent & WXUNUSED(event))
{
//
// current map click is: ZoomOut
//
  wxMenuBar *menuBar = GetMenuBar();
  wxToolBar *toolBar = GetToolBar();
  if (IsZoomOut == false)
    {
      if (menuBar)
        menuBar->Check(ID_ZoomOut, true);
      if (toolBar)
        toolBar->ToggleTool(ID_ZoomOut, true);
      IsIdentify = false;
      IsDistance = false;
      IsZoomIn = false;
      IsZoomOut = true;
      IsPan = false;
    }
}

void MyFrame::OnPan(wxCommandEvent & WXUNUSED(event))
{
//
// current map click is: Pan
//
  wxMenuBar *menuBar = GetMenuBar();
  wxToolBar *toolBar = GetToolBar();
  if (IsPan == false)
    {
      if (menuBar)
        menuBar->Check(ID_Pan, true);
      if (toolBar)
        toolBar->ToggleTool(ID_Pan, true);
      IsIdentify = false;
      IsDistance = false;
      IsZoomIn = false;
      IsZoomOut = false;
      IsPan = true;
    }
}

void MyFrame::SetMapInfo(int type)
{
// setting the current map description
  switch (type)
    {
      case ATLAS_SATELLITE:
        MapInfo = wxT("Satellite map");
        break;
      case ATLAS_RELIEF:
        MapInfo = wxT("Relief map");
        break;
      case ATLAS_BATHYMETRY:
        MapInfo = wxT("Bathymetry map");
        break;
      case ATLAS_TEMPERATURE_JAN:
        MapInfo = wxT("Mean Temperature: January");
        break;
      case ATLAS_TEMPERATURE_APR:
        MapInfo = wxT("Mean Temperature: April");
        break;
      case ATLAS_TEMPERATURE_JUL:
        MapInfo = wxT("Mean Temperature: July");
        break;
      case ATLAS_TEMPERATURE_OCT:
        MapInfo = wxT("Mean Temperature: October");
        break;
      case ATLAS_RAIN_JAN:
        MapInfo = wxT("Precipitation (rain): January");
        break;
      case ATLAS_RAIN_APR:
        MapInfo = wxT("Precipitation (rain): April");
        break;
      case ATLAS_RAIN_JUL:
        MapInfo = wxT("Precipitation (rain): July");
        break;
      case ATLAS_RAIN_OCT:
        MapInfo = wxT("Precipitation (rain): October");
        break;
      default:
        MapInfo = wxT("Political map");
        break;
    };
}

void MyFrame::UpdateMapInfo()
{
//
// updating the status bar [MapInfo]
//
  if (GetStatusBar() == NULL)
    return;
  GetStatusBar()->SetStatusText(MapInfo, 1);
}

void MyFrame::UpdateCoords(wxString & coords)
{
//
// updating the status bar [Coordinates]
//
  if (GetStatusBar() == NULL)
    return;
  GetStatusBar()->SetStatusText(coords, 2);
}

void MyFrame::SaveConfig()
{
//
// saves layout configuration
//

  wxConfig *config = new wxConfig(wxT("LibreAtlas"));
  config->Write(wxT("SqlitePath"), SqlitePath);
  delete config;
}

void MyFrame::LoadConfig()
{
//
// loads layout configuration
//
  int current_map = -1;
  ConfigDbPath = wxT("");
  wxConfig *config = new wxConfig(wxT("LibreAtlas"));
  config->Read(wxT("SqlitePath"), &ConfigDbPath);
  delete config;
  if (ConfigDbPath.Len() > 0)
    {
      SqlitePath = ConfigDbPath;
      if (OpenDB() == false)
        SqlitePath = wxT("");
      else
        {
          wxMenuBar *menuBar = GetMenuBar();
          menuBar->Enable(wxID_OPEN, false);
          menuBar->Enable(wxID_CLOSE, true);
          if (HasCountries() == true || HasAdminAreas() == true)
            {
              menuBar->Enable(ID_Political, true);
              if (current_map < 0)
                {
                  current_map = ID_Political;
                  IsPolitical = true;
                }
            }
          if (SatelliteHandle)
            {
              menuBar->Enable(ID_Satellite, true);
              if (current_map < 0)
                {
                  current_map = ID_Satellite;
                  IsSatellite = true;
                }
            }
          if (ReliefHandle)
            {
              menuBar->Enable(ID_Relief, true);
              if (current_map < 0)
                {
                  current_map = ID_Relief;
                  IsRelief = true;
                }
            }
          if (BathymetryHandle)
            {
              menuBar->Enable(ID_Bathymetry, true);
              if (current_map < 0)
                {
                  current_map = ID_Bathymetry;
                  IsBathymetry = true;
                }
            }
          if (TempJanHandle && TempAprHandle && TempJulHandle && TempOctHandle)
            {
              menuBar->Enable(ID_Temperature, true);
              if (current_map < 0)
                {
                  current_map = ID_Temperature;
                  IsTemperature = true;
                }
            }
          if (RainJanHandle && RainAprHandle && RainJulHandle && RainOctHandle)
            {
              menuBar->Enable(ID_Rain, true);
              if (current_map < 0)
                {
                  current_map = ID_Rain;
                  IsRain = true;
                }
            }
          menuBar->Enable(ID_January, false);
          menuBar->Enable(ID_April, false);
          menuBar->Enable(ID_July, false);
          menuBar->Enable(ID_October, false);
          menuBar->Enable(ID_Center, true);
          menuBar->Enable(ID_BestZoom, false);
          menuBar->Enable(ID_Identify, HasIdentify());
          menuBar->Enable(ID_Distance, true);
          menuBar->Enable(ID_ZoomIn, true);
          menuBar->Enable(ID_ZoomOut, true);
          menuBar->Enable(ID_Pan, true);
          menuBar->Enable(ID_Graticule, HasGraticule());
          if (current_map > 0)
            menuBar->Check(current_map, true);
          menuBar->Check(ID_ZoomIn, true);
          IsGraticule = false;
          menuBar->Check(ID_Graticule, false);
          wxToolBar *toolBar = GetToolBar();
          toolBar->EnableTool(wxID_OPEN, false);
          toolBar->EnableTool(wxID_CLOSE, true);
          if (HasCountries() == true || HasAdminAreas() == true)
            toolBar->EnableTool(ID_Political, true);
          if (SatelliteHandle)
            toolBar->EnableTool(ID_Satellite, true);
          if (ReliefHandle)
            toolBar->EnableTool(ID_Relief, true);
          if (BathymetryHandle)
            toolBar->EnableTool(ID_Bathymetry, true);
          if (TempJanHandle && TempAprHandle && TempJulHandle && TempOctHandle)
            toolBar->EnableTool(ID_Temperature, true);
          if (RainJanHandle && RainAprHandle && RainJulHandle && RainOctHandle)
            toolBar->EnableTool(ID_Rain, true);
          if ((TempJanHandle && TempAprHandle && TempJulHandle
               && TempOctHandle) || (RainJanHandle && RainAprHandle
                                     && RainJulHandle && RainOctHandle))
            toolBar->EnableTool(ID_Month, false);
          toolBar->EnableTool(ID_Center, true);
          toolBar->EnableTool(ID_BestZoom, false);
          toolBar->EnableTool(ID_Identify, HasIdentify());
          toolBar->EnableTool(ID_Distance, true);
          toolBar->EnableTool(ID_ZoomIn, true);
          toolBar->EnableTool(ID_ZoomOut, true);
          toolBar->EnableTool(ID_Pan, true);
          toolBar->EnableTool(ID_Graticule, HasGraticule());
          if (current_map > 0)
            toolBar->ToggleTool(current_map, true);
          toolBar->ToggleTool(ID_ZoomIn, true);
          UpdateStatusBar();
          MapView->SetFullExtent();
          MapView->PrepareMap();
        }
    }
}

void MyFrame::OnOpen(wxCommandEvent & WXUNUSED(event))
{
//
// opening an ATLAS
//
  int current_map = -1;
  int ret;
  wxString lastDir;
  wxFileDialog fileDialog(this, wxT("Open Atlas"),
                          wxT(""), wxT("LibreAtlas.atlas"),
                          wxT
                          ("LibreAtlas (*.atlas)|*.atlas|All files (*.*)|*.*"),
                          wxFD_OPEN | wxFD_FILE_MUST_EXIST, wxDefaultPosition,
                          wxDefaultSize, wxT("filedlg"));
  ret = fileDialog.ShowModal();
  if (ret == wxID_OK)
    {
      SqlitePath = fileDialog.GetPath();
      if (OpenDB() == false)
        SqlitePath = wxT("");
      else
        {
          wxMenuBar *menuBar = GetMenuBar();
          menuBar->Enable(wxID_OPEN, false);
          menuBar->Enable(wxID_CLOSE, true);
          if (HasCountries() == true || HasAdminAreas() == true)
            {
              menuBar->Enable(ID_Political, true);
              if (current_map < 0)
                {
                  current_map = ID_Political;
                  IsPolitical = true;
                }
            }
          if (SatelliteHandle)
            {
              menuBar->Enable(ID_Satellite, true);
              if (current_map < 0)
                {
                  current_map = ID_Satellite;
                  IsSatellite = true;
                }
            }
          if (ReliefHandle)
            {
              menuBar->Enable(ID_Relief, true);
              if (current_map < 0)
                {
                  current_map = ID_Relief;
                  IsRelief = true;
                }
            }
          if (BathymetryHandle)
            {
              menuBar->Enable(ID_Bathymetry, true);
              if (current_map < 0)
                {
                  current_map = ID_Bathymetry;
                  IsBathymetry = true;
                }
            }
          if (TempJanHandle && TempAprHandle && TempJulHandle && TempOctHandle)
            {
              menuBar->Enable(ID_Temperature, true);
              if (current_map < 0)
                {
                  current_map = ID_Temperature;
                  IsTemperature = true;
                }
            }
          if (RainJanHandle && RainAprHandle && RainJulHandle && RainOctHandle)
            {
              menuBar->Enable(ID_Rain, true);
              if (current_map < 0)
                {
                  current_map = ID_Rain;
                  IsRain = true;
                }
            }
          menuBar->Enable(ID_January, false);
          menuBar->Enable(ID_April, false);
          menuBar->Enable(ID_July, false);
          menuBar->Enable(ID_October, false);
          menuBar->Enable(ID_Center, true);
          menuBar->Enable(ID_BestZoom, false);
          menuBar->Enable(ID_Identify, HasIdentify());
          menuBar->Enable(ID_Distance, true);
          menuBar->Enable(ID_ZoomIn, true);
          menuBar->Enable(ID_ZoomOut, true);
          menuBar->Enable(ID_Pan, true);
          menuBar->Enable(ID_Graticule, HasGraticule());
          if (current_map > 0)
            menuBar->Check(current_map, true);
          menuBar->Check(ID_ZoomIn, true);
          IsGraticule = false;
          menuBar->Check(ID_Graticule, false);
          wxToolBar *toolBar = GetToolBar();
          toolBar->EnableTool(wxID_OPEN, false);
          toolBar->EnableTool(wxID_CLOSE, true);
          if (HasCountries() == true || HasAdminAreas() == true)
            toolBar->EnableTool(ID_Political, true);
          if (SatelliteHandle)
            toolBar->EnableTool(ID_Satellite, true);
          if (ReliefHandle)
            toolBar->EnableTool(ID_Relief, true);
          if (BathymetryHandle)
            toolBar->EnableTool(ID_Bathymetry, true);
          if (TempJanHandle && TempAprHandle && TempJulHandle && TempOctHandle)
            toolBar->EnableTool(ID_Temperature, true);
          if (RainJanHandle && RainAprHandle && RainJulHandle && RainOctHandle)
            toolBar->EnableTool(ID_Rain, true);
          if ((TempJanHandle && TempAprHandle && TempJulHandle
               && TempOctHandle) || (RainJanHandle && RainAprHandle
                                     && RainJulHandle && RainOctHandle))
            toolBar->EnableTool(ID_Month, false);
          toolBar->EnableTool(ID_Center, true);
          toolBar->EnableTool(ID_BestZoom, false);
          toolBar->EnableTool(ID_Identify, HasIdentify());
          toolBar->EnableTool(ID_Distance, true);
          toolBar->EnableTool(ID_ZoomIn, true);
          toolBar->EnableTool(ID_ZoomOut, true);
          toolBar->EnableTool(ID_Pan, true);
          toolBar->EnableTool(ID_Graticule, HasGraticule());
          if (current_map > 0)
            toolBar->ToggleTool(current_map, true);
          toolBar->ToggleTool(ID_ZoomIn, true);
          MapView->SetFullExtent();
          MapView->PrepareMap();
        }
    }
  UpdateStatusBar();
}

void MyFrame::OnClose(wxCommandEvent & WXUNUSED(event))
{
//
// closing the current ATLAS
//
  CloseDB();
  wxMenuBar *menuBar = GetMenuBar();
  menuBar->Enable(wxID_OPEN, true);
  menuBar->Enable(wxID_CLOSE, false);
  menuBar->Enable(ID_Political, false);
  menuBar->Enable(ID_Satellite, false);
  menuBar->Enable(ID_Relief, false);
  menuBar->Enable(ID_Bathymetry, false);
  menuBar->Enable(ID_Temperature, false);
  menuBar->Enable(ID_Rain, false);
  menuBar->Enable(ID_January, false);
  menuBar->Enable(ID_April, false);
  menuBar->Enable(ID_July, false);
  menuBar->Enable(ID_October, false);
  menuBar->Enable(ID_Center, false);
  menuBar->Enable(ID_BestZoom, false);
  menuBar->Enable(ID_Identify, false);
  menuBar->Enable(ID_Distance, false);
  menuBar->Enable(ID_ZoomIn, false);
  menuBar->Enable(ID_ZoomOut, false);
  menuBar->Enable(ID_Pan, false);
  menuBar->Enable(ID_Graticule, false);
  wxToolBar *toolBar = GetToolBar();
  toolBar->EnableTool(wxID_OPEN, true);
  toolBar->EnableTool(wxID_CLOSE, false);
  toolBar->EnableTool(ID_Political, false);
  toolBar->EnableTool(ID_Satellite, false);
  toolBar->EnableTool(ID_Relief, false);
  toolBar->EnableTool(ID_Bathymetry, false);
  toolBar->EnableTool(ID_Temperature, false);
  toolBar->EnableTool(ID_Rain, false);
  toolBar->EnableTool(ID_Month, false);
  toolBar->EnableTool(ID_Center, false);
  toolBar->EnableTool(ID_BestZoom, false);
  toolBar->EnableTool(ID_Identify, false);
  toolBar->EnableTool(ID_Distance, false);
  toolBar->EnableTool(ID_ZoomIn, false);
  toolBar->EnableTool(ID_ZoomOut, false);
  toolBar->EnableTool(ID_Pan, false);
  toolBar->EnableTool(ID_Graticule, false);
  UpdateStatusBar();
  MapView->Invalidate();
  IsPolitical = false;
  IsSatellite = false;
  IsRelief = false;
  IsBathymetry = false;
  IsTemperature = false;
  IsRain = false;
  IsZoomIn = true;
  IsZoomOut = false;
  IsPan = false;
  IsGraticule = false;
  IsIdentify = false;
  IsDistance = false;
  IsZoomIn = true;
  IsZoomOut = false;
  IsPan = false;
}

void MyFrame::UpdateStatusBar()
{
//
// updating the status bar
//
  if (GetStatusBar() == NULL)
    return;
  if (SqlitePath.Len() < 1)
    GetStatusBar()->SetStatusText(wxT("No Atlas"), 0);
  else
    GetStatusBar()->SetStatusText(wxT("Current Atlas: ") + SqlitePath, 0);
}

bool MyFrame::CheckCountriesLayout(const char *table_name)
{
//
// checking the Countries table for validity
//
  int ret;
  char **results;
  int rows;
  int columns;
  int i;
  wxString sql;
  const char *name;
  const char *type;
  const char *pk;
  bool ok_pkey = false;
  bool ok_continent = false;
  bool ok_name = false;
  bool ok_capital = false;
  bool ok_area_km2 = false;
  bool ok_population = false;
  bool ok_currency_code = false;
  bool ok_currency_name = false;
  bool ok_pop_km2 = false;
  bool ok_color = false;
  bool ok_flag = false;
  bool ok_emblem = false;

  sql = wxT("PRAGMA table_info(");
  sql += wxString::FromUTF8(table_name);
  sql += wxT(")");
  ret = sqlite3_get_table(GetSqlite(), sql.ToUTF8(), &results,
                          &rows, &columns, NULL);
  if (ret != SQLITE_OK)
    goto error;
  if (rows < 1)
    ;
  else
    {
      for (i = 1; i <= rows; i++)
        {
          name = results[(i * columns) + 1];
          type = results[(i * columns) + 2];
          pk = results[(i * columns) + 5];
          if (strcmp(name, "code") == 0 && strcmp(type, "TEXT") == 0
              && strcmp(pk, "1") == 0)
            ok_pkey = true;
          if (strcmp(name, "name") == 0 && strcmp(type, "TEXT") == 0)
            ok_name = true;
          if (strcmp(name, "continent") == 0 && strcmp(type, "TEXT") == 0)
            ok_continent = true;
          if (strcmp(name, "capital") == 0 && strcmp(type, "TEXT") == 0)
            ok_capital = true;
          if (strcmp(name, "area_km2") == 0 && strcmp(type, "DOUBLE") == 0)
            ok_area_km2 = true;
          if (strcmp(name, "population") == 0 && strcmp(type, "INTEGER") == 0)
            ok_population = true;
          if (strcmp(name, "currency_code") == 0 && strcmp(type, "TEXT") == 0)
            ok_currency_code = true;
          if (strcmp(name, "currency_name") == 0 && strcmp(type, "TEXT") == 0)
            ok_currency_name = true;
          if (strcmp(name, "pop_km2") == 0 && strcmp(type, "DOUBLE") == 0)
            ok_pop_km2 = true;
          if (strcmp(name, "color") == 0 && strcmp(type, "TEXT") == 0)
            ok_color = true;
          if (strcmp(name, "flag") == 0 && strcmp(type, "BLOB") == 0)
            ok_flag = true;
          if (strcmp(name, "emblem") == 0 && strcmp(type, "BLOB") == 0)
            ok_emblem = true;
        }
    }
  sqlite3_free_table(results);

  if (ok_pkey && ok_name && ok_continent && ok_capital && ok_area_km2
      && ok_population && ok_currency_code && ok_currency_name & ok_pop_km2
      && ok_color && ok_flag && ok_emblem)
    return true;

error:
  return false;
}

bool MyFrame::CheckGeographicLinesLayout(const char *table_name)
{
//
// checking the GeographicLines table for validity
//
  int ret;
  char **results;
  int rows;
  int columns;
  int i;
  wxString sql;
  const char *name;
  const char *type;
  const char *pk;
  bool ok_pkey = false;
  bool ok_name = false;
  bool ok_note = false;

  sql = wxT("PRAGMA table_info(");
  sql += wxString::FromUTF8(table_name);
  sql += wxT(")");
  ret = sqlite3_get_table(GetSqlite(), sql.ToUTF8(), &results,
                          &rows, &columns, NULL);
  if (ret != SQLITE_OK)
    goto error;
  if (rows < 1)
    ;
  else
    {
      for (i = 1; i <= rows; i++)
        {
          name = results[(i * columns) + 1];
          type = results[(i * columns) + 2];
          pk = results[(i * columns) + 5];
          if (strcmp(name, "pk_uid") == 0 && strcmp(type, "INTEGER") == 0
              && strcmp(pk, "1") == 0)
            ok_pkey = true;
          if (strcmp(name, "name") == 0 && strcmp(type, "TEXT") == 0)
            ok_name = true;
          if (strcmp(name, "note") == 0 && strcmp(type, "TEXT") == 0)
            ok_note = true;
        }
    }
  sqlite3_free_table(results);

  if (ok_pkey && ok_name && ok_note)
    return true;

error:
  return false;
}

bool MyFrame::CheckGraticulesLayout(const char *table_name)
{
//
// checking the Graticules table for validity
//
  int ret;
  char **results;
  int rows;
  int columns;
  int i;
  wxString sql;
  const char *name;
  const char *type;
  const char *pk;
  bool ok_pkey = false;
  bool ok_type = false;
  bool ok_label = false;
  bool ok_class = false;

  sql = wxT("PRAGMA table_info(");
  sql += wxString::FromUTF8(table_name);
  sql += wxT(")");
  ret = sqlite3_get_table(GetSqlite(), sql.ToUTF8(), &results,
                          &rows, &columns, NULL);
  if (ret != SQLITE_OK)
    goto error;
  if (rows < 1)
    ;
  else
    {
      for (i = 1; i <= rows; i++)
        {
          name = results[(i * columns) + 1];
          type = results[(i * columns) + 2];
          pk = results[(i * columns) + 5];
          if (strcmp(name, "pk_uid") == 0 && strcmp(type, "INTEGER") == 0
              && strcmp(pk, "1") == 0)
            ok_pkey = true;
          if (strcmp(name, "type") == 0 && strcmp(type, "TEXT") == 0)
            ok_type = true;
          if (strcmp(name, "label") == 0 && strcmp(type, "TEXT") == 0)
            ok_label = true;
          if (strcmp(name, "class") == 0 && strcmp(type, "INTEGER") == 0)
            ok_class = true;
        }
    }
  sqlite3_free_table(results);

  if (ok_pkey && ok_type && ok_label && ok_class)
    return true;

error:
  return false;
}

bool MyFrame::CheckTownsLayout(const char *table_name)
{
//
// checking the Towns table for validity
//
  int ret;
  char **results;
  int rows;
  int columns;
  int i;
  wxString sql;
  const char *name;
  const char *type;
  const char *pk;
  bool ok_pkey = false;
  bool ok_name = false;
  bool ok_population = false;
  bool ok_type = false;

  sql = wxT("PRAGMA table_info(");
  sql += wxString::FromUTF8(table_name);
  sql += wxT(")");
  ret = sqlite3_get_table(GetSqlite(), sql.ToUTF8(), &results,
                          &rows, &columns, NULL);
  if (ret != SQLITE_OK)
    goto error;
  if (rows < 1)
    ;
  else
    {
      for (i = 1; i <= rows; i++)
        {
          name = results[(i * columns) + 1];
          type = results[(i * columns) + 2];
          pk = results[(i * columns) + 5];
          if (strcmp(name, "pk_uid") == 0 && strcmp(type, "INTEGER") == 0
              && strcmp(pk, "1") == 0)
            ok_pkey = true;
          if (strcmp(name, "name") == 0 && strcmp(type, "TEXT") == 0)
            ok_name = true;
          if (strcmp(name, "population") == 0 && strcmp(type, "INTEGER") == 0)
            ok_population = true;
          if (strcmp(name, "type") == 0 && strcmp(type, "TEXT") == 0)
            ok_type = true;
        }
    }
  sqlite3_free_table(results);

  if (ok_pkey && ok_name && ok_population && ok_type)
    return true;

error:
  return false;
}

bool MyFrame::CheckAdminAreasLayout(const char *table_name)
{
//
// checking the AdminAreas table for validity
//
  int ret;
  char **results;
  int rows;
  int columns;
  int i;
  wxString sql;
  const char *name;
  const char *type;
  const char *pk;
  bool ok_pkey = false;
  bool ok_name = false;
  bool ok_class = false;
  bool ok_color = false;
  bool ok_coat_of_arms = false;

  sql = wxT("PRAGMA table_info(");
  sql += wxString::FromUTF8(table_name);
  sql += wxT(")");
  ret = sqlite3_get_table(GetSqlite(), sql.ToUTF8(), &results,
                          &rows, &columns, NULL);
  if (ret != SQLITE_OK)
    goto error;
  if (rows < 1)
    ;
  else
    {
      for (i = 1; i <= rows; i++)
        {
          name = results[(i * columns) + 1];
          type = results[(i * columns) + 2];
          pk = results[(i * columns) + 5];
          if (strcmp(name, "pk_uid") == 0 && strcmp(type, "INTEGER") == 0
              && strcmp(pk, "1") == 0)
            ok_pkey = true;
          if (strcmp(name, "name") == 0 && strcmp(type, "TEXT") == 0)
            ok_name = true;
          if (strcmp(name, "class") == 0 && strcmp(type, "TEXT") == 0)
            ok_class = true;
          if (strcmp(name, "color") == 0 && strcmp(type, "TEXT") == 0)
            ok_color = true;
          if (strcmp(name, "coat_of_arms") == 0 && strcmp(type, "BLOB") == 0)
            ok_coat_of_arms = true;
        }
    }
  sqlite3_free_table(results);

  if (ok_pkey && ok_name && ok_class && ok_color && ok_coat_of_arms)
    return true;

error:
  return false;
}

bool MyFrame::CheckRailwaysLayout(const char *table_name)
{
//
// checking the Railways table for validity
//
  int ret;
  char **results;
  int rows;
  int columns;
  int i;
  wxString sql;
  const char *name;
  const char *type;
  const char *pk;
  bool ok_pkey = false;
  bool ok_name = false;
  bool ok_tracks = false;

  sql = wxT("PRAGMA table_info(");
  sql += wxString::FromUTF8(table_name);
  sql += wxT(")");
  ret = sqlite3_get_table(GetSqlite(), sql.ToUTF8(), &results,
                          &rows, &columns, NULL);
  if (ret != SQLITE_OK)
    goto error;
  if (rows < 1)
    ;
  else
    {
      for (i = 1; i <= rows; i++)
        {
          name = results[(i * columns) + 1];
          type = results[(i * columns) + 2];
          pk = results[(i * columns) + 5];
          if (strcmp(name, "pk_uid") == 0 && strcmp(type, "INTEGER") == 0
              && strcmp(pk, "1") == 0)
            ok_pkey = true;
          if (strcmp(name, "name") == 0 && strcmp(type, "TEXT") == 0)
            ok_name = true;
          if (strcmp(name, "tracks") == 0 && strcmp(type, "INTEGER") == 0)
            ok_tracks = true;
        }
    }
  sqlite3_free_table(results);

  if (ok_pkey && ok_name && ok_tracks)
    return true;

error:
  return false;
}

bool MyFrame::CheckGenericLayout(const char *table_name)
{
//
// checking a generic table for validity
//
  int ret;
  char **results;
  int rows;
  int columns;
  int i;
  wxString sql;
  const char *name;
  const char *type;
  const char *pk;
  bool ok_pkey = false;
  bool ok_name = false;

  sql = wxT("PRAGMA table_info(");
  sql += wxString::FromUTF8(table_name);
  sql += wxT(")");
  ret = sqlite3_get_table(GetSqlite(), sql.ToUTF8(), &results,
                          &rows, &columns, NULL);
  if (ret != SQLITE_OK)
    goto error;
  if (rows < 1)
    ;
  else
    {
      for (i = 1; i <= rows; i++)
        {
          name = results[(i * columns) + 1];
          type = results[(i * columns) + 2];
          pk = results[(i * columns) + 5];
          if (strcmp(name, "pk_uid") == 0 && strcmp(type, "INTEGER") == 0
              && strcmp(pk, "1") == 0)
            ok_pkey = true;
          if (strcmp(name, "name") == 0 && strcmp(type, "TEXT") == 0)
            ok_name = true;
        }
    }
  sqlite3_free_table(results);

  if (ok_pkey && ok_name)
    return true;

error:
  return false;
}

bool MyFrame::CheckGenericWithClassLayout(const char *table_name)
{
//
// checking a generic table (with Class) for validity
//
  int ret;
  char **results;
  int rows;
  int columns;
  int i;
  wxString sql;
  const char *name;
  const char *type;
  const char *pk;
  bool ok_pkey = false;
  bool ok_name = false;
  bool ok_class = false;

  sql = wxT("PRAGMA table_info(");
  sql += wxString::FromUTF8(table_name);
  sql += wxT(")");
  ret = sqlite3_get_table(GetSqlite(), sql.ToUTF8(), &results,
                          &rows, &columns, NULL);
  if (ret != SQLITE_OK)
    goto error;
  if (rows < 1)
    ;
  else
    {
      for (i = 1; i <= rows; i++)
        {
          name = results[(i * columns) + 1];
          type = results[(i * columns) + 2];
          pk = results[(i * columns) + 5];
          if (strcmp(name, "pk_uid") == 0 && strcmp(type, "INTEGER") == 0
              && strcmp(pk, "1") == 0)
            ok_pkey = true;
          if (strcmp(name, "name") == 0 && strcmp(type, "TEXT") == 0)
            ok_name = true;
          if (strcmp(name, "class") == 0 && strcmp(type, "TEXT") == 0)
            ok_class = true;
        }
    }
  sqlite3_free_table(results);

  if (ok_pkey && ok_name && ok_class)
    return true;

error:
  return false;
}

bool MyFrame::SetCoordinatesLongLat()
{
//
// checking if current CRS is long/lat or projected
//
  int ret;
  char **results;
  int rows;
  int columns;
  wxString sql;
  char dummy[64];

  UseLongLatCoords = false;
  WorldWideMap = false;

  sql = wxT("SELECT * FROM spatial_ref_sys ");
  sql += wxT("WHERE proj4text like '%+proj=longlat%' AND srid = ");
  sprintf(dummy, "%d", Srid);
  sql += wxString::FromUTF8(dummy);
  ret = sqlite3_get_table(GetSqlite(), sql.ToUTF8(), &results,
                          &rows, &columns, NULL);
  if (ret != SQLITE_OK)
    goto error;
  if (rows < 1)
    ;
  else
    UseLongLatCoords = true;
  sqlite3_free_table(results);
  return true;

error:
  return false;
}

void MyFrame::GetSRSName(wxString & srs)
{
//
// reporting the currently used SRS
//
  int ret;
  char **results;
  int rows;
  int columns;
  int i;
  wxString sql;
  char dummy[64];
  const char *auth;
  const char *name;

  srs = wxT("Unknown/Undefined Spatial Reference System");

  sql = wxT("SELECT auth_name, ref_sys_name FROM spatial_ref_sys ");
  sql += wxT("WHERE srid = ");
  sprintf(dummy, "%d", Srid);
  sql += wxString::FromUTF8(dummy);
  ret = sqlite3_get_table(GetSqlite(), sql.ToUTF8(), &results,
                          &rows, &columns, NULL);
  if (ret != SQLITE_OK)
    goto error;
  if (rows < 1)
    ;
  else
    {
      for (i = 1; i <= rows; i++)
        {
          auth = results[(i * columns) + 0];
          name = results[(i * columns) + 1];
          srs = wxT("Currently used Spatial Reference System is:\n");
          srs += wxString::FromUTF8(auth) + wxT(" SRID=");
          srs += wxString::FromUTF8(dummy) + wxT(" \"");
          srs += wxString::FromUTF8(name) + wxT("\"");
          if (IsGeographic() == true)
            srs +=
              wxT
              ("\nit's a GEOGRAPHIC SRS, based on Latitude and Longitude angles");
          else
            srs +=
              wxT
              ("\nit's a PLANAR PROJECTED SRS, using metric cartesian coordinates");
        }
    }
  sqlite3_free_table(results);

error:
  return;
}

bool MyFrame::ComputeMapExtent()
{
//
// computing the full Map Extent
//
  int ret;
  wxString sql;
  sqlite3_stmt *stmt;

  sql = wxT("SELECT Min(extent_min_x), Min(extent_min_y), ");
  sql += wxT("Max(extent_max_x), Max(extent_max_y) ");
  sql += wxT("FROM layer_statistics");
  ret = sqlite3_prepare_v2(SqliteHandle, sql.ToUTF8(), sql.Len(), &stmt, NULL);
  if (ret != SQLITE_OK)
    {
      wxString err = wxString::FromUTF8(sqlite3_errmsg(SqliteHandle));
      wxMessageBox(wxT("SQL error:") + err, wxT("LibreAtlas"),
                   wxOK | wxICON_ERROR, this);
      return false;
    }

  while (1)
    {
      // scrolling the result set 
      ret = sqlite3_step(stmt);
      if (ret == SQLITE_DONE)
        break;                  // end of result set
      if (ret == SQLITE_ROW)
        {
          MapMinX = sqlite3_column_double(stmt, 0);
          MapMinY = sqlite3_column_double(stmt, 1);
          MapMaxX = sqlite3_column_double(stmt, 2);
          MapMaxY = sqlite3_column_double(stmt, 3);
          if (UseLongLatCoords == true)
            {
              if ((MapMaxX - MapMinX) > 90.0 || (MapMaxY - MapMinY) > 45.0)
                WorldWideMap = true;
            }
      } else
        {
          wxString err = wxString::FromUTF8(sqlite3_errmsg(SqliteHandle));
          wxMessageBox(wxT("SQL error:") + err, wxT("LibreAtlas"),
                       wxOK | wxICON_ERROR, this);
          sqlite3_finalize(stmt);
          return false;
        }
    }
  sqlite3_finalize(stmt);
  return true;
}

double MyFrame::GetRasterMaxResolution(double current)
{
//
// computing the Raster full resolution zoom level
//
  int ret;
  wxString sql;
  sqlite3_stmt *stmt;
  double resolution = current;
  wxString name;

// setting the current Raster datasource name
  if (IsSatellite)
    name = SatellitePrefix;
  if (IsRelief)
    name = ReliefPrefix;
  if (IsBathymetry)
    name = BathymetryPrefix;
  if (IsTemperature)
    {
      switch (CurrentMonth)
        {
          case 1:
            name = TempJanPrefix;
            break;
          case 4:
            name = TempAprPrefix;
            break;
          case 7:
            name = TempJulPrefix;
            break;
          case 10:
            name = TempOctPrefix;
            break;
        };
    }
  if (IsRain)
    {
      switch (CurrentMonth)
        {
          case 1:
            name = RainJanPrefix;
            break;
          case 4:
            name = RainAprPrefix;
            break;
          case 7:
            name = RainJulPrefix;
            break;
          case 10:
            name = RainOctPrefix;
            break;
        };
    }
  if (name.Len() == 0)
    return resolution;

  sql = wxT("SELECT Min(pixel_x_size) FROM raster_pyramids ");
  sql += wxT("WHERE table_prefix = '") + name + wxT("'");
  ret = sqlite3_prepare_v2(SqliteHandle, sql.ToUTF8(), sql.Len(), &stmt, NULL);
  if (ret != SQLITE_OK)
    {
      wxString err = wxString::FromUTF8(sqlite3_errmsg(SqliteHandle));
      wxMessageBox(wxT("SQL error:") + err, wxT("LibreAtlas"),
                   wxOK | wxICON_ERROR, this);
      return resolution;
    }

  while (1)
    {
      // scrolling the result set 
      ret = sqlite3_step(stmt);
      if (ret == SQLITE_DONE)
        break;                  // end of result set
      if (ret == SQLITE_ROW)
        {
          resolution = sqlite3_column_double(stmt, 0);
      } else
        {
          wxString err = wxString::FromUTF8(sqlite3_errmsg(SqliteHandle));
          wxMessageBox(wxT("SQL error:") + err, wxT("LibreAtlas"),
                       wxOK | wxICON_ERROR, this);
          sqlite3_finalize(stmt);
          return resolution;
        }
    }
  sqlite3_finalize(stmt);
  return resolution;
}

bool MyFrame::CheckDB()
{
//
// checking the DB for validity
//
  int ret;
  char **results;
  int rows;
  int columns;
  int i;
  wxString sql;
  const char *table;
  const char *geom;
  const char *type;
  const char *srid;
  const char *rtree;
  bool invalid = false;
  bool ok_temp = false;
  bool ok_rain = false;
  bool ok_vector = false;
  bool ok_identify = false;
  int refSrid = -1;
  int sridSatellite = -1;
  int sridRelief = -1;
  int sridBathymetry = -1;
  int sridTempJan = -1;
  int sridTempApr = -1;
  int sridTempJul = -1;
  int sridTempOct = -1;
  int sridRainJan = -1;
  int sridRainApr = -1;
  int sridRainJul = -1;
  int sridRainOct = -1;
  int sridCountries = -1;
  int sridGeographicLines = -1;
  int sridGraticules = -1;
  int sridLakes = -1;
  int sridLandAreas = -1;
  int sridMarineAreas = -1;
  int sridRivers = -1;
  int sridTowns = -1;
  int sridAdminAreas = -1;
  int sridCulturalHeritage = -1;
  int sridNames = -1;
  int sridHighways = -1;
  int sridNaturalReserves = -1;
  int sridRailwayStations = -1;
  int sridRailways = -1;
  int sridTownAreas = -1;
  SatellitePrefix = wxT("");
  ReliefPrefix = wxT("");
  BathymetryPrefix = wxT("");
  TempJanPrefix = wxT("");
  TempAprPrefix = wxT("");
  TempJulPrefix = wxT("");
  TempOctPrefix = wxT("");
  RainJanPrefix = wxT("");
  RainAprPrefix = wxT("");
  RainJulPrefix = wxT("");
  RainOctPrefix = wxT("");
  CountriesTable = wxT("");
  GeographicLinesTable = wxT("");
  GraticulesTable = wxT("");
  LakesTable = wxT("");
  LandAreasTable = wxT("");
  MarineAreasTable = wxT("");
  RiversTable = wxT("");
  TownsTable = wxT("");
  AdminAreasTable = wxT("");
  CulturalHeritageTable = wxT("");
  NamesTable = wxT("");
  HighwaysTable = wxT("");
  NaturalReservesTable = wxT("");
  RailwayStationsTable = wxT("");
  RailwaysTable = wxT("");
  TownAreasTable = wxT("");
  Srid = -1;
  UseLongLatCoords = true;
  OkGraticule = false;
  IdentifyCountry = false;
  IdentifyAdminArea = false;

  sql = wxT("SELECT f_table_name, f_geometry_column, type, ");
  sql += wxT("srid, spatial_index_enabled FROM geometry_columns");
  ret = sqlite3_get_table(GetSqlite(), sql.ToUTF8(), &results,
                          &rows, &columns, NULL);
  if (ret != SQLITE_OK)
    goto error;
  if (rows < 1)
    ;
  else
    {
      for (i = 1; i <= rows; i++)
        {
          table = results[(i * columns) + 0];
          geom = results[(i * columns) + 1];
          type = results[(i * columns) + 2];
          srid = results[(i * columns) + 3];
          rtree = results[(i * columns) + 4];
          if (table && geom && type && srid && rtree)
            ;
          else
            continue;
          if (strcmp(table, "TrueMarble_metadata") == 0 &&
              strcmp(geom, "geometry") == 0 &&
              strcmp(type, "POLYGON") == 0 && strcmp(rtree, "1") == 0)
            {
              SatellitePrefix = wxString::FromUTF8("TrueMarble");
              sridSatellite = atoi(srid);
              refSrid = atoi(srid);
            }
          if (strcmp(table, "landsat_metadata") == 0 &&
              strcmp(geom, "geometry") == 0 &&
              strcmp(type, "POLYGON") == 0 && strcmp(rtree, "1") == 0)
            {
              SatellitePrefix = wxString::FromUTF8("landsat");
              sridSatellite = atoi(srid);
              refSrid = atoi(srid);
            }
          if (strcmp(table, "Clementine_metadata") == 0 &&
              strcmp(geom, "geometry") == 0 &&
              strcmp(type, "POLYGON") == 0 && strcmp(rtree, "1") == 0)
            {
              SatellitePrefix = wxString::FromUTF8("Clementine");
              sridSatellite = atoi(srid);
              refSrid = atoi(srid);
            }
          if (strcmp(table, "srtm30_metadata") == 0 &&
              strcmp(geom, "geometry") == 0 &&
              strcmp(type, "POLYGON") == 0 && strcmp(rtree, "1") == 0)
            {
              ReliefPrefix = wxString::FromUTF8("srtm30");
              sridRelief = atoi(srid);
              refSrid = atoi(srid);
            }
          if (strcmp(table, "srtm_metadata") == 0 &&
              strcmp(geom, "geometry") == 0 &&
              strcmp(type, "POLYGON") == 0 && strcmp(rtree, "1") == 0)
            {
              ReliefPrefix = wxString::FromUTF8("srtm");
              sridRelief = atoi(srid);
              refSrid = atoi(srid);
            }
          if (strcmp(table, "noaa_metadata") == 0 &&
              strcmp(geom, "geometry") == 0 &&
              strcmp(type, "POLYGON") == 0 && strcmp(rtree, "1") == 0)
            {
              BathymetryPrefix = wxString::FromUTF8("noaa");
              sridBathymetry = atoi(srid);
              refSrid = atoi(srid);
            }
          if (strcmp(table, "tmean_jan_metadata") == 0 &&
              strcmp(geom, "geometry") == 0 &&
              strcmp(type, "POLYGON") == 0 && strcmp(rtree, "1") == 0)
            {
              TempJanPrefix = wxString::FromUTF8("tmean_jan");
              sridTempJan = atoi(srid);
            }
          if (strcmp(table, "tmean_apr_metadata") == 0 &&
              strcmp(geom, "geometry") == 0 &&
              strcmp(type, "POLYGON") == 0 && strcmp(rtree, "1") == 0)
            {
              TempAprPrefix = wxString::FromUTF8("tmean_apr");
              sridTempApr = atoi(srid);
            }
          if (strcmp(table, "tmean_jul_metadata") == 0 &&
              strcmp(geom, "geometry") == 0 &&
              strcmp(type, "POLYGON") == 0 && strcmp(rtree, "1") == 0)
            {
              TempJulPrefix = wxString::FromUTF8("tmean_jul");
              sridTempJul = atoi(srid);
            }
          if (strcmp(table, "tmean_oct_metadata") == 0 &&
              strcmp(geom, "geometry") == 0 &&
              strcmp(type, "POLYGON") == 0 && strcmp(rtree, "1") == 0)
            {
              TempOctPrefix = wxString::FromUTF8("tmean_oct");
              sridTempOct = atoi(srid);
            }
          if (strcmp(table, "prec_jan_metadata") == 0 &&
              strcmp(geom, "geometry") == 0 &&
              strcmp(type, "POLYGON") == 0 && strcmp(rtree, "1") == 0)
            {
              RainJanPrefix = wxString::FromUTF8("prec_jan");
              sridRainJan = atoi(srid);
            }
          if (strcmp(table, "prec_apr_metadata") == 0 &&
              strcmp(geom, "geometry") == 0 &&
              strcmp(type, "POLYGON") == 0 && strcmp(rtree, "1") == 0)
            {
              RainAprPrefix = wxString::FromUTF8("prec_apr");
              sridRainApr = atoi(srid);
            }
          if (strcmp(table, "prec_jul_metadata") == 0 &&
              strcmp(geom, "geometry") == 0 &&
              strcmp(type, "POLYGON") == 0 && strcmp(rtree, "1") == 0)
            {
              RainJulPrefix = wxString::FromUTF8("prec_jul");
              sridRainJul = atoi(srid);
            }
          if (strcmp(table, "prec_oct_metadata") == 0 &&
              strcmp(geom, "geometry") == 0 &&
              strcmp(type, "POLYGON") == 0 && strcmp(rtree, "1") == 0)
            {
              RainOctPrefix = wxString::FromUTF8("prec_oct");
              sridRainOct = atoi(srid);
            }
          if (strcmp(table, "countries") == 0 &&
              strcmp(geom, "geom") == 0 &&
              (strcmp(type, "MULTIPOLYGON") == 0
               || strcmp(type, "POLYGON") == 0) && strcmp(rtree, "1") == 0)
            {
              if (CheckCountriesLayout("countries"))
                {
                  CountriesTable = wxT("countries");
                  sridCountries = atoi(srid);
                  ok_identify = true;
                  ok_vector = true;
                }
            }
          if (strcmp(table, "geographic_lines") == 0 &&
              strcmp(geom, "geom") == 0 &&
              (strcmp(type, "MULTILINESTRING") == 0
               || strcmp(type, "LINESTRING") == 0) && strcmp(rtree, "1") == 0)
            {
              if (CheckGeographicLinesLayout("geographic_lines"))
                {
                  GeographicLinesTable = wxT("geographic_lines");
                  sridGeographicLines = atoi(srid);
                  OkGraticule = true;
                }
            }
          if (strcmp(table, "graticules") == 0 &&
              strcmp(geom, "geom") == 0 &&
              (strcmp(type, "MULTILINESTRING") == 0
               || strcmp(type, "LINESTRING") == 0) && strcmp(rtree, "1") == 0)
            {
              if (CheckGraticulesLayout("graticules"))
                {
                  GraticulesTable = wxT("graticules");
                  sridGraticules = atoi(srid);
                  OkGraticule = true;
                }
            }
          if (strcmp(table, "lakes") == 0 &&
              strcmp(geom, "geom") == 0 &&
              (strcmp(type, "MULTIPOLYGON") == 0
               || strcmp(type, "POLYGON") == 0) && strcmp(rtree, "1") == 0)
            {
              if (CheckGenericLayout("lakes"))
                {
                  LakesTable = wxT("lakes");
                  sridLakes = atoi(srid);
                  ok_vector = true;
                }
            }
          if (strcmp(table, "land_areas") == 0 &&
              strcmp(geom, "geom") == 0 &&
              (strcmp(type, "MULTIPOLYGON") == 0
               || strcmp(type, "POLYGON") == 0) && strcmp(rtree, "1") == 0)
            {
              if (CheckGenericWithClassLayout("land_areas"))
                {
                  LandAreasTable = wxT("land_areas");
                  sridLandAreas = atoi(srid);
                  ok_vector = true;
                }
            }
          if (strcmp(table, "marine_areas") == 0 &&
              strcmp(geom, "geom") == 0 &&
              (strcmp(type, "MULTIPOLYGON") == 0
               || strcmp(type, "POLYGON") == 0) && strcmp(rtree, "1") == 0)
            {
              if (CheckGenericLayout("marine_areas"))
                {
                  MarineAreasTable = wxT("marine_areas");
                  sridMarineAreas = atoi(srid);
                  ok_vector = true;
                }
            }
          if (strcmp(table, "rivers") == 0 &&
              strcmp(geom, "geom") == 0 &&
              (strcmp(type, "MULTILINESTRING") == 0
               || strcmp(type, "LINESTRING") == 0) && strcmp(rtree, "1") == 0)
            {
              if (CheckGenericLayout("rivers"))
                {
                  RiversTable = wxT("rivers");
                  sridRivers = atoi(srid);
                  ok_vector = true;
                }
            }
          if (strcmp(table, "towns") == 0 &&
              strcmp(geom, "geom") == 0 &&
              strcmp(type, "POINT") == 0 && strcmp(rtree, "1") == 0)
            {
              if (CheckTownsLayout("towns"))
                {
                  TownsTable = wxT("towns");
                  sridTowns = atoi(srid);
                  ok_vector = true;
                }
            }
          if (strcmp(table, "admin_areas") == 0 &&
              strcmp(geom, "geom") == 0 &&
              (strcmp(type, "POLYGON") == 0
               || strcmp(type, "MULTIPOLYGON") == 0) && strcmp(rtree, "1") == 0)
            {
              if (CheckAdminAreasLayout("admin_areas"))
                {
                  AdminAreasTable = wxT("admin_areas");
                  sridAdminAreas = atoi(srid);
                  ok_vector = true;
                  ok_identify = true;
                }
            }
          if (strcmp(table, "cultural_heritage") == 0 &&
              strcmp(geom, "geom") == 0 &&
              strcmp(type, "POINT") == 0 && strcmp(rtree, "1") == 0)
            {
              if (CheckGenericLayout("cultural_heritage"))
                {
                  CulturalHeritageTable = wxT("cultural_heritage");
                  sridCulturalHeritage = atoi(srid);
                  ok_vector = true;
                }
            }
          if (strcmp(table, "names") == 0 &&
              strcmp(geom, "geom") == 0 &&
              strcmp(type, "POINT") == 0 && strcmp(rtree, "1") == 0)
            {
              if (CheckGenericWithClassLayout("names"))
                {
                  NamesTable = wxT("names");
                  sridNames = atoi(srid);
                  ok_vector = true;
                }
            }
          if (strcmp(table, "highways") == 0 &&
              strcmp(geom, "geom") == 0 &&
              (strcmp(type, "MULTILINESTRING") == 0
               || strcmp(type, "LINESTRING") == 0) && strcmp(rtree, "1") == 0)
            {
              if (CheckGenericWithClassLayout("highways"))
                {
                  HighwaysTable = wxT("highways");
                  sridHighways = atoi(srid);
                  ok_vector = true;
                }
            }
          if (strcmp(table, "natural_reserves") == 0 &&
              strcmp(geom, "geom") == 0 &&
              (strcmp(type, "POLYGON") == 0
               || strcmp(type, "MULTIPOLYGON") == 0) && strcmp(rtree, "1") == 0)
            {
              if (CheckGenericLayout("natural_reserves"))
                {
                  NaturalReservesTable = wxT("natural_reserves");
                  sridNaturalReserves = atoi(srid);
                  ok_vector = true;
                }
            }
          if (strcmp(table, "railway_stations") == 0 &&
              strcmp(geom, "geom") == 0 &&
              strcmp(type, "POINT") == 0 && strcmp(rtree, "1") == 0)
            {
              if (CheckGenericLayout("railway_stations"))
                {
                  RailwayStationsTable = wxT("railway_stations");
                  sridRailwayStations = atoi(srid);
                  ok_vector = true;
                }
            }
          if (strcmp(table, "railways") == 0 &&
              strcmp(geom, "geom") == 0 &&
              (strcmp(type, "MULTILINESTRING") == 0
               || strcmp(type, "LINESTRING") == 0) && strcmp(rtree, "1") == 0)
            {
              if (CheckRailwaysLayout("railways"))
                {
                  RailwaysTable = wxT("railways");
                  sridRailways = atoi(srid);
                  ok_vector = true;
                }
            }
          if (strcmp(table, "town_areas") == 0 &&
              strcmp(geom, "geom") == 0 &&
              (strcmp(type, "POLYGON") == 0
               || strcmp(type, "MULTIPOLYGON") == 0) && strcmp(rtree, "1") == 0)
            {
              if (CheckTownsLayout("town_areas"))
                {
                  TownAreasTable = wxT("town_areas");
                  sridTownAreas = atoi(srid);
                  ok_vector = true;
                }
            }
        }
    }
  sqlite3_free_table(results);

// checking Temperature RasterLite datasource
  if (TempJanPrefix.Len() > 0 && TempAprPrefix.Len() > 0
      && TempJulPrefix.Len() > 0 && TempOctPrefix.Len() > 0)
    {
      if (sridTempJan == sridTempApr && sridTempJan == sridTempJul
          && sridTempJan == sridTempOct)
        {
          ok_temp = true;
          refSrid = sridTempJan;
        }
    }
// checking Rain RasterLite datasource
  if (RainJanPrefix.Len() > 0 && RainAprPrefix.Len() > 0
      && RainJulPrefix.Len() > 0 && RainOctPrefix.Len() > 0)
    {
      if (sridRainJan == sridRainApr && sridRainJan == sridRainJul
          && sridRainJan == sridRainOct)
        {
          ok_rain = true;
          refSrid = sridRainJan;
        }
    }
// at least one RasterLite datasource is expected to be found
  if (SatellitePrefix.Len() == 0 && ReliefPrefix.Len() == 0
      && BathymetryPrefix.Len() == 0 && ok_temp == false && ok_rain == false)
    invalid = true;

  if (invalid == false)
    {
      // checking if all RasterLite datasources belongs to the same SRID
      if (SatellitePrefix.Len() > 0)
        {
          if (refSrid != sridSatellite)
            invalid = true;
        }
      if (ReliefPrefix.Len() > 0)
        {
          if (refSrid != sridRelief)
            invalid = true;
        }
      if (BathymetryPrefix.Len() > 0)
        {
          if (refSrid != sridBathymetry)
            invalid = true;
        }
      if (ok_temp == true)
        {
          if (refSrid != sridTempJan)
            invalid = true;
        }
      if (SatellitePrefix.Len() > 0)
        {
          if (refSrid != sridSatellite)
            invalid = true;
        }
    }
// at least one SpatiaLite (vector) layer is expected to be found
  if (ok_vector == false)
    invalid = true;
  else
    {
      // checking if all vector layers belongs to the same SRID of raster(s)
      if (CountriesTable.Len() > 0)
        {
          if (refSrid != sridCountries)
            invalid = true;
        }
      if (GeographicLinesTable.Len() > 0)
        {
          if (refSrid != sridGeographicLines)
            invalid = true;
        }
      if (GraticulesTable.Len() > 0)
        {
          if (refSrid != sridGraticules)
            invalid = true;
        }
      if (LakesTable.Len() > 0)
        {
          if (refSrid != sridLakes)
            invalid = true;
        }
      if (LandAreasTable.Len() > 0)
        {
          if (refSrid != sridLandAreas)
            invalid = true;
        }
      if (MarineAreasTable.Len() > 0)
        {
          if (refSrid != sridMarineAreas)
            invalid = true;
        }
      if (RiversTable.Len() > 0)
        {
          if (refSrid != sridRivers)
            invalid = true;
        }
      if (TownsTable.Len() > 0)
        {
          if (refSrid != sridTowns)
            invalid = true;
        }
      if (AdminAreasTable.Len() > 0)
        {
          if (refSrid != sridAdminAreas)
            invalid = true;
        }
      if (CulturalHeritageTable.Len() > 0)
        {
          if (refSrid != sridCulturalHeritage)
            invalid = true;
        }
      if (NamesTable.Len() > 0)
        {
          if (refSrid != sridNames)
            invalid = true;
        }
      if (HighwaysTable.Len() > 0)
        {
          if (refSrid != sridHighways)
            invalid = true;
        }
      if (NaturalReservesTable.Len() > 0)
        {
          if (refSrid != sridNaturalReserves)
            invalid = true;
        }
      if (RailwayStationsTable.Len() > 0)
        {
          if (refSrid != sridRailwayStations)
            invalid = true;
        }
      if (RailwaysTable.Len() > 0)
        {
          if (refSrid != sridRailways)
            invalid = true;
        }
      if (TownAreasTable.Len() > 0)
        {
          if (refSrid != sridTownAreas)
            invalid = true;
        }
      if (CountriesTable.Len() > 0)
        {
          IdentifyCountry = true;
          IdentifyAdminArea = false;
        }
      if (AdminAreasTable.Len() > 0)
        {
          IdentifyCountry = false;
          IdentifyAdminArea = true;
        }
    }

/* at least one Identify (vector) layer is expected to be found
  if (ok_identify == false)
    invalid = true;
*/

  Srid = refSrid;
  if (!SetCoordinatesLongLat())
    invalid = true;
  if (!ComputeMapExtent())
    invalid = true;

  if (invalid == false)
    return true;

error:
  wxMessageBox(wxT("this DB does not contain a valid Atlas:\n") + SqlitePath,
               wxT("LibreAtlas"), wxOK | wxICON_ERROR, this);
  return false;
}

bool MyFrame::OpenDB()
{
//
// establishing a physical connection to some DB SQLite
//
  int ret;
  char *errMsg = NULL;

  spatialite_init(0);           // loading the SpatiaLite extension

  ret =
    sqlite3_open_v2(SqlitePath.ToUTF8(), &SqliteHandle, SQLITE_OPEN_READONLY,
                    NULL);
  if (ret)
    {
      // an error occurred
      wxString errCause;
      errCause = wxString::FromUTF8(sqlite3_errmsg(SqliteHandle));
      sqlite3_close(SqliteHandle);
      wxMessageBox(wxT("Failure while connecting to DB\n\n") + errCause +
                   wxT("\n") + SqlitePath, wxT("LibreAtlas"),
                   wxOK | wxICON_ERROR, this);
      SqliteHandle = NULL;
      return false;
    }
// activating Foreign Key constraints
  ret = sqlite3_exec(SqliteHandle, "PRAGMA foreign_keys = 1", NULL, 0, &errMsg);
  if (ret != SQLITE_OK)
    {
      wxMessageBox(wxT("Unable to activate FOREIGN_KEY constraints"),
                   wxT("LibreAtlas"), wxOK | wxICON_INFORMATION, this);
      sqlite3_free(errMsg);
      SqliteHandle = NULL;
      return false;
    }
  if (!CheckDB())
    return false;
  if (SatellitePrefix.Len() > 0)
    OpenSatellite();
  if (ReliefPrefix.Len() > 0)
    OpenRelief();
  if (BathymetryPrefix.Len() > 0)
    OpenBathymetry();
  if (TempJanPrefix.Len() > 0 && TempAprPrefix.Len() > 0
      && TempJulPrefix.Len() > 0 && TempOctPrefix.Len() > 0)
    {
      OpenTempJan();
      OpenTempApr();
      OpenTempJul();
      OpenTempOct();
    }
  if (RainJanPrefix.Len() > 0 && RainAprPrefix.Len() > 0
      && RainJulPrefix.Len() > 0 && RainOctPrefix.Len() > 0)
    {
      OpenRainJan();
      OpenRainApr();
      OpenRainJul();
      OpenRainOct();
    }
  return true;
}

void MyFrame::OpenSatellite()
{
// opening the Satellite rasterlite datasource
  char db_path[1024];
  char source_name[256];
  void *handle;
  strcpy(db_path, SqlitePath.ToUTF8());
  strcpy(source_name, SatellitePrefix.ToUTF8());
  handle = rasterliteOpen(db_path, source_name);
  if (rasterliteIsError(handle))
    {
      rasterliteClose(handle);
      return;
    }
  rasterliteSetTransparentColor(handle, 0, 0, 0);
  rasterliteSetBackgroundColor(handle, 0, 0, 0);
  SatelliteHandle = handle;
}

void MyFrame::OpenRelief()
{
// opening the Relief rasterlite datasource
  char db_path[1024];
  char source_name[256];
  void *handle;
  strcpy(db_path, SqlitePath.ToUTF8());
  strcpy(source_name, ReliefPrefix.ToUTF8());
  handle = rasterliteOpen(db_path, source_name);
  if (rasterliteIsError(handle))
    {
      rasterliteClose(handle);
      return;
    }
  rasterliteSetTransparentColor(handle, 0, 0, 0);
  rasterliteSetBackgroundColor(handle, 0, 0, 0);
  ReliefHandle = handle;
}

void MyFrame::OpenBathymetry()
{
// opening the Bathymetry rasterlite datasource
  char db_path[1024];
  char source_name[256];
  void *handle;
  strcpy(db_path, SqlitePath.ToUTF8());
  strcpy(source_name, BathymetryPrefix.ToUTF8());
  handle = rasterliteOpen(db_path, source_name);
  if (rasterliteIsError(handle))
    {
      rasterliteClose(handle);
      return;
    }
  rasterliteSetTransparentColor(handle, 0, 0, 0);
  rasterliteSetBackgroundColor(handle, 0, 0, 0);
  BathymetryHandle = handle;
}

void MyFrame::OpenTempJan()
{
// opening the TempJan rasterlite datasource
  char db_path[1024];
  char source_name[256];
  void *handle;
  strcpy(db_path, SqlitePath.ToUTF8());
  strcpy(source_name, TempJanPrefix.ToUTF8());
  handle = rasterliteOpen(db_path, source_name);
  if (rasterliteIsError(handle))
    {
      rasterliteClose(handle);
      return;
    }
  rasterliteSetTransparentColor(handle, 0, 0, 0);
  rasterliteSetBackgroundColor(handle, 0, 0, 0);
  TempJanHandle = handle;
}

void MyFrame::OpenTempApr()
{
// opening the TempApr rasterlite datasource
  char db_path[1024];
  char source_name[256];
  void *handle;
  strcpy(db_path, SqlitePath.ToUTF8());
  strcpy(source_name, TempAprPrefix.ToUTF8());
  handle = rasterliteOpen(db_path, source_name);
  if (rasterliteIsError(handle))
    {
      rasterliteClose(handle);
      return;
    }
  rasterliteSetTransparentColor(handle, 0, 0, 0);
  rasterliteSetBackgroundColor(handle, 0, 0, 0);
  TempAprHandle = handle;
}

void MyFrame::OpenTempJul()
{
// opening the TempJul rasterlite datasource
  char db_path[1024];
  char source_name[256];
  void *handle;
  strcpy(db_path, SqlitePath.ToUTF8());
  strcpy(source_name, TempJulPrefix.ToUTF8());
  handle = rasterliteOpen(db_path, source_name);
  if (rasterliteIsError(handle))
    {
      rasterliteClose(handle);
      return;
    }
  rasterliteSetTransparentColor(handle, 0, 0, 0);
  rasterliteSetBackgroundColor(handle, 0, 0, 0);
  TempJulHandle = handle;
}

void MyFrame::OpenTempOct()
{
// opening the TempOct rasterlite datasource
  char db_path[1024];
  char source_name[256];
  void *handle;
  strcpy(db_path, SqlitePath.ToUTF8());
  strcpy(source_name, TempOctPrefix.ToUTF8());
  handle = rasterliteOpen(db_path, source_name);
  if (rasterliteIsError(handle))
    {
      rasterliteClose(handle);
      return;
    }
  rasterliteSetTransparentColor(handle, 0, 0, 0);
  rasterliteSetBackgroundColor(handle, 0, 0, 0);
  TempOctHandle = handle;
}

void MyFrame::OpenRainJan()
{
// opening the RainJan rasterlite datasource
  char db_path[1024];
  char source_name[256];
  void *handle;
  strcpy(db_path, SqlitePath.ToUTF8());
  strcpy(source_name, RainJanPrefix.ToUTF8());
  handle = rasterliteOpen(db_path, source_name);
  if (rasterliteIsError(handle))
    {
      rasterliteClose(handle);
      return;
    }
  rasterliteSetTransparentColor(handle, 0, 0, 0);
  rasterliteSetBackgroundColor(handle, 0, 0, 0);
  RainJanHandle = handle;
}

void MyFrame::OpenRainApr()
{
// opening the RainApr rasterlite datasource
  char db_path[1024];
  char source_name[256];
  void *handle;
  strcpy(db_path, SqlitePath.ToUTF8());
  strcpy(source_name, RainAprPrefix.ToUTF8());
  handle = rasterliteOpen(db_path, source_name);
  if (rasterliteIsError(handle))
    {
      rasterliteClose(handle);
      return;
    }
  rasterliteSetTransparentColor(handle, 0, 0, 0);
  rasterliteSetBackgroundColor(handle, 0, 0, 0);
  RainAprHandle = handle;
}

void MyFrame::OpenRainJul()
{
// opening the RainJul rasterlite datasource
  char db_path[1024];
  char source_name[256];
  void *handle;
  strcpy(db_path, SqlitePath.ToUTF8());
  strcpy(source_name, RainJulPrefix.ToUTF8());
  handle = rasterliteOpen(db_path, source_name);
  if (rasterliteIsError(handle))
    {
      rasterliteClose(handle);
      return;
    }
  rasterliteSetTransparentColor(handle, 0, 0, 0);
  rasterliteSetBackgroundColor(handle, 0, 0, 0);
  RainJulHandle = handle;
}

void MyFrame::OpenRainOct()
{
// opening the RainOct rasterlite datasource
  char db_path[1024];
  char source_name[256];
  void *handle;
  strcpy(db_path, SqlitePath.ToUTF8());
  strcpy(source_name, RainOctPrefix.ToUTF8());
  handle = rasterliteOpen(db_path, source_name);
  if (rasterliteIsError(handle))
    {
      rasterliteClose(handle);
      return;
    }
  rasterliteSetTransparentColor(handle, 0, 0, 0);
  rasterliteSetBackgroundColor(handle, 0, 0, 0);
  RainOctHandle = handle;
}

void MyFrame::CloseDB()
{
//
// disconnecting current SQLite DB
//
  if (SatelliteHandle)
    {
      rasterliteClose(SatelliteHandle);
      SatelliteHandle = NULL;
    }
  if (ReliefHandle)
    {
      rasterliteClose(ReliefHandle);
      ReliefHandle = NULL;
    }
  if (BathymetryHandle)
    {
      rasterliteClose(BathymetryHandle);
      BathymetryHandle = NULL;
    }
  if (TempJanHandle)
    {
      rasterliteClose(TempJanHandle);
      TempJanHandle = NULL;
    }
  if (TempAprHandle)
    {
      rasterliteClose(TempAprHandle);
      TempAprHandle = NULL;
    }
  if (TempJulHandle)
    {
      rasterliteClose(TempJulHandle);
      TempJulHandle = NULL;
    }
  if (TempOctHandle)
    {
      rasterliteClose(TempOctHandle);
      TempOctHandle = NULL;
    }
  if (RainJanHandle)
    {
      rasterliteClose(RainJanHandle);
      RainJanHandle = NULL;
    }
  if (RainAprHandle)
    {
      rasterliteClose(RainAprHandle);
      RainAprHandle = NULL;
    }
  if (RainJulHandle)
    {
      rasterliteClose(RainJulHandle);
      RainJulHandle = NULL;
    }
  if (RainOctHandle)
    {
      rasterliteClose(RainOctHandle);
      RainOctHandle = NULL;
    }
  if (!SqliteHandle)
    return;
  sqlite3_close(SqliteHandle);
  SqliteHandle = NULL;
  SqlitePath = wxT("");
  MapView->Invalidate();
  MapView->Refresh();
  MapInfo = wxT("");
  UpdateStatusBar();
  UpdateMapInfo();
  Refresh();
}

void MyFrame::OnQuit(wxCommandEvent & WXUNUSED(event))
{
//
// EXIT - event handler
//
  CloseDB();
  Close(true);
}

void MyFrame::OnAbout(wxCommandEvent & WXUNUSED(event))
{
//
// ABOUT dialog - event handler
//
  char ver[128];
  wxAboutDialogInfo dlg;
  dlg.SetIcon(wxIcon(icon_info_xpm));
  dlg.SetName(wxT("LibreAtlas"));
  const char *version = VERSION;
  dlg.SetVersion(wxString::FromUTF8(version));
  wxString str = wxT("a free Atlas viewer\n\n");
  sprintf(ver, "%d.%d.%d", wxMAJOR_VERSION, wxMINOR_VERSION, wxRELEASE_NUMBER);
  str += wxT("wxWidgets version ") + wxString::FromUTF8(ver) + wxT("\n");
  strcpy(ver, spatialite_version());
  str += wxT("SpatiaLite version ") + wxString::FromUTF8(ver) + wxT("\n");
  strcpy(ver, rasterliteGetVersion());
  str += wxT("RasterLite version ") + wxString::FromUTF8(ver) + wxT("\n");
  strcpy(ver, sqlite3_libversion());
  str += wxT("SQLite version ") + wxString::FromUTF8(ver) + wxT("\n");
  strcpy(ver, GEOSversion());
  str += wxT("GEOS version ") + wxString::FromUTF8(ver) + wxT("\n");
  strcpy(ver, pj_get_release());
  str += wxT("PROJ.4 version ") + wxString::FromUTF8(ver) + wxT("\n\n");
  if (MapView->IsValidMap() == true)
    {
      wxString srs;
      GetSRSName(srs);
      str += srs;
      str += wxT("\n\n");
    }
  dlg.SetDescription(str);
  dlg.SetCopyright(wxT("by Alessandro Furieri - 2012"));
  dlg.SetWebSite(wxT("http://www.gaia-gis.it"));
  wxString license =
    wxT("This program is free software; you can redistribute it\n");
  license +=
    wxT("and/or modify it under the terms of the GNU General Public License\n");
  license += wxT("(GPL) as published by the Free Software Foundation\n\n");
  license +=
    wxT
    ("A copy of the GPL can be found at\nhttp://www.gnu.org/licenses/gpl.txt");
  dlg.SetLicense(license);
  ::wxAboutBox(dlg);
}

bool MyFrame::IsTMeanJanMap()
{
  if (IsTemperature == true && CurrentMonth == 1)
    return true;
  return false;
}

bool MyFrame::IsTMeanAprMap()
{
  if (IsTemperature == true && CurrentMonth == 4)
    return true;
  return false;
}

bool MyFrame::IsTMeanJulMap()
{
  if (IsTemperature == true && CurrentMonth == 7)
    return true;
  return false;
}

bool MyFrame::IsTMeanOctMap()
{
  if (IsTemperature == true && CurrentMonth == 10)
    return true;
  return false;
}

bool MyFrame::IsRainJanMap()
{
  if (IsRain == true && CurrentMonth == 1)
    return true;
  return false;
}

bool MyFrame::IsRainAprMap()
{
  if (IsRain == true && CurrentMonth == 4)
    return true;
  return false;
}

bool MyFrame::IsRainJulMap()
{
  if (IsRain == true && CurrentMonth == 7)
    return true;
  return false;
}

bool MyFrame::IsRainOctMap()
{
  if (IsRain == true && CurrentMonth == 10)
    return true;
  return false;
}
