/*
/ MapHelpers.cpp
/ misc methods supporting the Map
/
/ version 1.0, 2012 June 8
/
/ Author: Sandro Furieri a-furieri@lqt.it
/
/ Copyright (C) 2012  Alessandro Furieri
/
/    This program is free software: you can redistribute it and/or modify
/    it under the terms of the GNU General Public License as published by
/    the Free Software Foundation, either version 3 of the License, or
/    (at your option) any later version.
/
/    This program is distributed in the hope that it will be useful,
/    but WITHOUT ANY WARRANTY; without even the implied warranty of
/    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
/    GNU General Public License for more details.
/
/    You should have received a copy of the GNU General Public License
/    along with this program.  If not, see <http://www.gnu.org/licenses/>.
/
*/

#include "Classdef.h"
#include "wx/mstream.h"

void MyMapView::DoIdentify(int mouse_x, int mouse_y)
{
//
// Identify [Country or AdminArea]
//
  if (MainFrame->IsIdentifyCountry() == true)
    DoIdentifyCountry(mouse_x, mouse_y);
  else if (MainFrame->IsIdentifyAdminArea() == true)
    DoIdentifyAdminArea(mouse_x, mouse_y);
}

void MyMapView::DoIdentifyCountry(int mouse_x, int mouse_y)
{
//
// Identify [Country info panel]
//
  sqlite3 *sqlite = MainFrame->GetSqlite();
  int ret;
  wxString sql;
  sqlite3_stmt *stmt;
  char xSql[1024];
  double queryMinX;
  double queryMaxX;
  double queryMinY;
  double queryMaxY;
  bool ok = false;
  wxString continent_code;
  wxString continent_name;
  wxString country_code;
  wxString country_name;
  wxString capital;
  double area_km2 = 0.0;
  int population;
  wxString currency_code;
  wxString currency_name;
  double pop_km2 = 0.0;
  wxBitmap flag;
  wxBitmap emblem;
  queryMinX = FrameMinX + ((double) (mouse_x - 4) * PixelRatio);
  queryMaxX = FrameMinX + ((double) (mouse_x + 4) * PixelRatio);
  queryMinY = FrameMaxY - ((double) (mouse_y + 4) * PixelRatio);
  queryMaxY = FrameMaxY - ((double) (mouse_y - 4) * PixelRatio);

// preparing the SQL statement [Countries]
  sql = wxT("SELECT b.code, b.name, a.code, a.name, a.capital, a.area_km2, ");
  sql += wxT("a.population, a.currency_code, a.currency_name, a.pop_km2, ");
  sql +=
    wxT("a.flag, a.emblem, a.geom FROM ") + MainFrame->GetCountries() +
    wxT(" AS a ");
  sql += wxT("LEFT JOIN continents AS b ON (a.continent = b.code) ");
  sql += wxT("WHERE Intersects(a.geom, BuildMbr(?, ?, ?, ?))");

  // using the R*Tree Spatial Index
  sql +=
    wxT(" AND a.ROWID IN (SELECT pkid FROM idx_") + MainFrame->GetCountries() +
    wxT("_geom");
  sql += wxT(" WHERE xmin <= ? AND xmax >= ? AND ymin <= ? AND ymax >= ?)");

  strcpy(xSql, sql.ToUTF8());
  ret = sqlite3_prepare_v2(sqlite, xSql, sql.Len(), &stmt, NULL);
  if (ret != SQLITE_OK)
    {
      wxString err = wxString::FromUTF8(sqlite3_errmsg(sqlite));
      wxMessageBox(wxT("SQL error:") + err, wxT("LibreAtlas"),
                   wxOK | wxICON_ERROR, this);
      goto stop;
    }
// query params binding
  sqlite3_reset(stmt);
  sqlite3_clear_bindings(stmt);
  sqlite3_bind_double(stmt, 1, queryMinX);
  sqlite3_bind_double(stmt, 2, queryMinY);
  sqlite3_bind_double(stmt, 3, queryMaxX);
  sqlite3_bind_double(stmt, 4, queryMaxY);

  // using the R*Tree Spatial Index
  sqlite3_bind_double(stmt, 5, queryMaxX);
  sqlite3_bind_double(stmt, 6, queryMinX);
  sqlite3_bind_double(stmt, 7, queryMaxY);
  sqlite3_bind_double(stmt, 8, queryMinY);

  while (1)
    {
      // scrolling the result set 
      ret = sqlite3_step(stmt);
      if (ret == SQLITE_DONE)
        break;                  // end of result set
      if (ret == SQLITE_ROW)
        {
          ok = true;
          if (sqlite3_column_type(stmt, 0) == SQLITE_TEXT)
            continent_code =
              wxString::FromUTF8((char *) sqlite3_column_text(stmt, 0));
          if (sqlite3_column_type(stmt, 1) == SQLITE_TEXT)
            continent_name =
              wxString::FromUTF8((char *) sqlite3_column_text(stmt, 1));
          country_code =
            wxString::FromUTF8((char *) sqlite3_column_text(stmt, 2));
          country_name =
            wxString::FromUTF8((char *) sqlite3_column_text(stmt, 3));
          if (sqlite3_column_type(stmt, 4) == SQLITE_TEXT)
            capital = wxString::FromUTF8((char *) sqlite3_column_text(stmt, 4));
          if (sqlite3_column_type(stmt, 5) == SQLITE_FLOAT)
            area_km2 = sqlite3_column_double(stmt, 5);
          if (sqlite3_column_type(stmt, 6) == SQLITE_INTEGER)
            population = sqlite3_column_int(stmt, 6);
          if (sqlite3_column_type(stmt, 7) == SQLITE_TEXT)
            currency_code =
              wxString::FromUTF8((char *) sqlite3_column_text(stmt, 7));
          if (sqlite3_column_type(stmt, 8) == SQLITE_TEXT)
            currency_name =
              wxString::FromUTF8((char *) sqlite3_column_text(stmt, 8));
          if (sqlite3_column_type(stmt, 9) == SQLITE_FLOAT)
            pop_km2 = sqlite3_column_double(stmt, 9);
          if (sqlite3_column_type(stmt, 10) == SQLITE_BLOB)
            {
              unsigned char *blob =
                (unsigned char *) sqlite3_column_blob(stmt, 10);
              int blob_size = sqlite3_column_bytes(stmt, 10);
              int blobType = gaiaGuessBlobType(blob, blob_size);
              if (blobType == GAIA_JPEG_BLOB
                  || blobType == GAIA_EXIF_BLOB
                  || blobType == GAIA_EXIF_GPS_BLOB
                  || blobType == GAIA_PNG_BLOB
                  || blobType == GAIA_GIF_BLOB || blobType == GAIA_TIFF_BLOB)
                {
                  wxMemoryInputStream reader(blob, blob_size);
                  wxImage *img = new wxImage(reader);
                  if (img->IsOk() == true)
                    {
                      int horz = img->GetWidth();
                      int vert = img->GetHeight();
                      while (horz > 128 || vert > 128)
                        {
                          horz *= 0.9;
                          vert *= 0.9;
                        }
                      wxImage scaledImg;
                      if (horz == img->GetWidth() && vert == img->GetHeight())
                        scaledImg = img->Copy();
                      else
                        scaledImg =
                          img->Scale((int) horz, (int) vert,
                                     wxIMAGE_QUALITY_HIGH);
                      flag = wxBitmap(scaledImg);
                    }
                  delete img;
                }
            }
          if (sqlite3_column_type(stmt, 11) == SQLITE_BLOB)
            {
              unsigned char *blob =
                (unsigned char *) sqlite3_column_blob(stmt, 11);
              int blob_size = sqlite3_column_bytes(stmt, 11);
              int blobType = gaiaGuessBlobType(blob, blob_size);
              if (blobType == GAIA_JPEG_BLOB
                  || blobType == GAIA_EXIF_BLOB
                  || blobType == GAIA_EXIF_GPS_BLOB
                  || blobType == GAIA_PNG_BLOB
                  || blobType == GAIA_GIF_BLOB || blobType == GAIA_TIFF_BLOB)
                {
                  wxMemoryInputStream reader(blob, blob_size);
                  wxImage *img = new wxImage(reader);
                  if (img->IsOk() == true)
                    {
                      int horz = img->GetWidth();
                      int vert = img->GetHeight();
                      while (horz > 128 || vert > 128)
                        {
                          horz *= 0.9;
                          vert *= 0.9;
                        }
                      wxImage scaledImg;
                      if (horz == img->GetWidth() && vert == img->GetHeight())
                        scaledImg = img->Copy();
                      else
                        scaledImg =
                          img->Scale((int) horz, (int) vert,
                                     wxIMAGE_QUALITY_HIGH);
                      emblem = wxBitmap(scaledImg);
                    }
                  delete img;
                }
            }
      } else
        {
          wxString err = wxString::FromUTF8(sqlite3_errmsg(sqlite));
          wxMessageBox(wxT("SQL error:") + err, wxT("LibreAtlas"),
                       wxOK | wxICON_ERROR, this);
          sqlite3_finalize(stmt);
          goto stop;
        }
    }
  sqlite3_finalize(stmt);

  if (ok == true)
    {
      // showing the info dialog
      CountryInfoDialog *dlg =
        new CountryInfoDialog(this, continent_code, continent_name,
                              country_code, country_name,
                              capital, currency_code, currency_name, area_km2,
                              population, pop_km2, flag, emblem);
      dlg->ShowModal();
      delete dlg;
    }

stop:
  return;
}

void MyMapView::DoIdentifyAdminArea(int mouse_x, int mouse_y)
{
//
// Identify [AdminArea info panel]
//
  sqlite3 *sqlite = MainFrame->GetSqlite();
  int ret;
  wxString sql;
  sqlite3_stmt *stmt;
  char xSql[1024];
  double queryMinX;
  double queryMaxX;
  double queryMinY;
  double queryMaxY;
  bool ok = false;
  wxString name;
  wxString type;
  wxBitmap coat_of_arms;
  wxString current_class = wxT("1");
  queryMinX = FrameMinX + ((double) (mouse_x - 4) * PixelRatio);
  queryMaxX = FrameMinX + ((double) (mouse_x + 4) * PixelRatio);
  queryMinY = FrameMaxY - ((double) (mouse_y + 4) * PixelRatio);
  queryMaxY = FrameMaxY - ((double) (mouse_y - 4) * PixelRatio);

  if (LayerIsVisible(ATLAS_ADMIN_AREAS_3) == true)
    current_class = wxT("3");
  if (LayerIsVisible(ATLAS_ADMIN_AREAS_2) == true)
    current_class = wxT("2");

// preparing the SQL statement [Countries]
  sql = wxT("SELECT name, class, coat_of_arms, geom ");
  sql += wxT("FROM ") + MainFrame->GetAdminAreas();
  sql += wxT(" WHERE class = '") + current_class;
  sql += wxT("' AND Intersects(geom, BuildMbr(?, ?, ?, ?))");

  // using the R*Tree Spatial Index
  sql +=
    wxT(" AND ROWID IN (SELECT pkid FROM idx_") + MainFrame->GetAdminAreas() +
    wxT("_geom");
  sql += wxT(" WHERE xmin <= ? AND xmax >= ? AND ymin <= ? AND ymax >= ?)");

  strcpy(xSql, sql.ToUTF8());
  ret = sqlite3_prepare_v2(sqlite, xSql, sql.Len(), &stmt, NULL);
  if (ret != SQLITE_OK)
    {
      wxString err = wxString::FromUTF8(sqlite3_errmsg(sqlite));
      wxMessageBox(wxT("SQL error:") + err, wxT("LibreAtlas"),
                   wxOK | wxICON_ERROR, this);
      goto stop;
    }
// query params binding
  sqlite3_reset(stmt);
  sqlite3_clear_bindings(stmt);
  sqlite3_bind_double(stmt, 1, queryMinX);
  sqlite3_bind_double(stmt, 2, queryMinY);
  sqlite3_bind_double(stmt, 3, queryMaxX);
  sqlite3_bind_double(stmt, 4, queryMaxY);

  // using the R*Tree Spatial Index
  sqlite3_bind_double(stmt, 5, queryMaxX);
  sqlite3_bind_double(stmt, 6, queryMinX);
  sqlite3_bind_double(stmt, 7, queryMaxY);
  sqlite3_bind_double(stmt, 8, queryMinY);

  while (1)
    {
      // scrolling the result set 
      ret = sqlite3_step(stmt);
      if (ret == SQLITE_DONE)
        break;                  // end of result set
      if (ret == SQLITE_ROW)
        {
          ok = true;
          if (sqlite3_column_type(stmt, 0) == SQLITE_TEXT)
            name = wxString::FromUTF8((char *) sqlite3_column_text(stmt, 0));
          if (sqlite3_column_type(stmt, 1) == SQLITE_TEXT)
            type = wxString::FromUTF8((char *) sqlite3_column_text(stmt, 1));
          if (sqlite3_column_type(stmt, 2) == SQLITE_BLOB)
            {
              unsigned char *blob =
                (unsigned char *) sqlite3_column_blob(stmt, 2);
              int blob_size = sqlite3_column_bytes(stmt, 2);
              int blobType = gaiaGuessBlobType(blob, blob_size);
              if (blobType == GAIA_JPEG_BLOB
                  || blobType == GAIA_EXIF_BLOB
                  || blobType == GAIA_EXIF_GPS_BLOB
                  || blobType == GAIA_PNG_BLOB
                  || blobType == GAIA_GIF_BLOB || blobType == GAIA_TIFF_BLOB)
                {
                  wxMemoryInputStream reader(blob, blob_size);
                  wxImage *img = new wxImage(reader);
                  if (img->IsOk() == true)
                    {
                      int horz = img->GetWidth();
                      int vert = img->GetHeight();
                      while (horz > 128 || vert > 128)
                        {
                          horz *= 0.9;
                          vert *= 0.9;
                        }
                      wxImage scaledImg;
                      if (horz == img->GetWidth() && vert == img->GetHeight())
                        scaledImg = img->Copy();
                      else
                        scaledImg =
                          img->Scale((int) horz, (int) vert,
                                     wxIMAGE_QUALITY_HIGH);
                      coat_of_arms = wxBitmap(scaledImg);
                    }
                  delete img;
                }
            }
      } else
        {
          wxString err = wxString::FromUTF8(sqlite3_errmsg(sqlite));
          wxMessageBox(wxT("SQL error:") + err, wxT("LibreAtlas"),
                       wxOK | wxICON_ERROR, this);
          sqlite3_finalize(stmt);
          goto stop;
        }
    }
  sqlite3_finalize(stmt);

  if (ok == true)
    {
      // showing the info dialog
      AdminAreaInfoDialog *dlg =
        new AdminAreaInfoDialog(this, name, type, coat_of_arms);
      dlg->ShowModal();
      delete dlg;
    }

stop:
  return;
}

void MyMapView::OnTheFlyIdentify(int mouse_x, int mouse_y)
{
//
// on-the-fly Identify [Country / Town / Rivers and so on]
//
  sqlite3 *sqlite = MainFrame->GetSqlite();
  int ret;
  wxString sql;
  sqlite3_stmt *stmt;
  char xSql[1024];
  char dummy[128];
  wxString LabelValue;
  double queryMinX;
  double queryMaxX;
  double queryMinY;
  double queryMaxY;
  sqlite3_int64 minId;
  queryMinX = FrameMinX + ((double) (mouse_x - 4) * PixelRatio);
  queryMaxX = FrameMinX + ((double) (mouse_x + 4) * PixelRatio);
  queryMinY = FrameMaxY - ((double) (mouse_y + 4) * PixelRatio);
  queryMaxY = FrameMaxY - ((double) (mouse_y - 4) * PixelRatio);

  if (LayerIsVisible(ATLAS_CULTURAL_HERITAGE) == true)
    {
// preparing the SQL statement [CulturalHeritage]
      sql = wxT("SELECT name, geom FROM ") + MainFrame->GetCulturalHeritage();
      sql += wxT(" WHERE Intersects(geom, BuildMbr(?, ?, ?, ?))");

      // using the R*Tree Spatial Index
      sql +=
        wxT(" AND ROWID IN (SELECT pkid FROM idx_") +
        MainFrame->GetCulturalHeritage() + wxT("_geom");
      sql += wxT(" WHERE xmin <= ? AND xmax >= ? AND ymin <= ? AND ymax >= ?)");

      strcpy(xSql, sql.ToUTF8());
      ret = sqlite3_prepare_v2(sqlite, xSql, sql.Len(), &stmt, NULL);
      if (ret != SQLITE_OK)
        {
          wxString err = wxString::FromUTF8(sqlite3_errmsg(sqlite));
          wxMessageBox(wxT("SQL error:") + err, wxT("LibreAtlas"),
                       wxOK | wxICON_ERROR, this);
          goto stop;
        }
// query params binding
      sqlite3_reset(stmt);
      sqlite3_clear_bindings(stmt);
      sqlite3_bind_double(stmt, 1, queryMinX);
      sqlite3_bind_double(stmt, 2, queryMinY);
      sqlite3_bind_double(stmt, 3, queryMaxX);
      sqlite3_bind_double(stmt, 4, queryMaxY);

      // using the R*Tree Spatial Index
      sqlite3_bind_double(stmt, 5, queryMaxX);
      sqlite3_bind_double(stmt, 6, queryMinX);
      sqlite3_bind_double(stmt, 7, queryMaxY);
      sqlite3_bind_double(stmt, 8, queryMinY);

      while (1)
        {
          // scrolling the result set 
          ret = sqlite3_step(stmt);
          if (ret == SQLITE_DONE)
            break;              // end of result set
          if (ret == SQLITE_ROW)
            {
              LabelValue =
                wxString::FromUTF8((char *) sqlite3_column_text(stmt, 0));
              const void *blob = sqlite3_column_blob(stmt, 1);
              int blob_size = sqlite3_column_bytes(stmt, 1);
              gaiaGeomCollPtr geom =
                gaiaFromSpatiaLiteBlobWkb((const unsigned char *) blob,
                                          blob_size);
              if (geom != NULL)
                SetCurrentGeometry(geom);
          } else
            {
              wxString err = wxString::FromUTF8(sqlite3_errmsg(sqlite));
              wxMessageBox(wxT("SQL error:") + err, wxT("LibreAtlas"),
                           wxOK | wxICON_ERROR, this);
              sqlite3_finalize(stmt);
              goto stop;
            }
        }
      sqlite3_finalize(stmt);
      if (LabelValue.Len() > 0)
        {
          ShowMapTip(LabelValue, mouse_x, mouse_y, ATLAS_CULTURAL_HERITAGE);
          return;
        }
    }

  if (LayerIsVisible(ATLAS_RAILWAY_STATIONS) == true)
    {
// preparing the SQL statement [RailwayStations]
      sql = wxT("SELECT name, geom FROM ") + MainFrame->GetRailwayStations();
      sql += wxT(" WHERE Intersects(geom, BuildMbr(?, ?, ?, ?))");

      // using the R*Tree Spatial Index
      sql +=
        wxT(" AND ROWID IN (SELECT pkid FROM idx_") +
        MainFrame->GetRailwayStations() + wxT("_geom");
      sql += wxT(" WHERE xmin <= ? AND xmax >= ? AND ymin <= ? AND ymax >= ?)");

      strcpy(xSql, sql.ToUTF8());
      ret = sqlite3_prepare_v2(sqlite, xSql, sql.Len(), &stmt, NULL);
      if (ret != SQLITE_OK)
        {
          wxString err = wxString::FromUTF8(sqlite3_errmsg(sqlite));
          wxMessageBox(wxT("SQL error:") + err, wxT("LibreAtlas"),
                       wxOK | wxICON_ERROR, this);
          goto stop;
        }
// query params binding
      sqlite3_reset(stmt);
      sqlite3_clear_bindings(stmt);
      sqlite3_bind_double(stmt, 1, queryMinX);
      sqlite3_bind_double(stmt, 2, queryMinY);
      sqlite3_bind_double(stmt, 3, queryMaxX);
      sqlite3_bind_double(stmt, 4, queryMaxY);

      // using the R*Tree Spatial Index
      sqlite3_bind_double(stmt, 5, queryMaxX);
      sqlite3_bind_double(stmt, 6, queryMinX);
      sqlite3_bind_double(stmt, 7, queryMaxY);
      sqlite3_bind_double(stmt, 8, queryMinY);

      while (1)
        {
          // scrolling the result set 
          ret = sqlite3_step(stmt);
          if (ret == SQLITE_DONE)
            break;              // end of result set
          if (ret == SQLITE_ROW)
            {
              LabelValue =
                wxString::FromUTF8((char *) sqlite3_column_text(stmt, 0));
              const void *blob = sqlite3_column_blob(stmt, 1);
              int blob_size = sqlite3_column_bytes(stmt, 1);
              gaiaGeomCollPtr geom =
                gaiaFromSpatiaLiteBlobWkb((const unsigned char *) blob,
                                          blob_size);
              if (geom != NULL)
                SetCurrentGeometry(geom);
          } else
            {
              wxString err = wxString::FromUTF8(sqlite3_errmsg(sqlite));
              wxMessageBox(wxT("SQL error:") + err, wxT("LibreAtlas"),
                           wxOK | wxICON_ERROR, this);
              sqlite3_finalize(stmt);
              goto stop;
            }
        }
      sqlite3_finalize(stmt);
      if (LabelValue.Len() > 0)
        {
          ShowMapTip(LabelValue, mouse_x, mouse_y, ATLAS_RAILWAY_STATIONS);
          return;
        }
    }

  if (LayerIsVisible(ATLAS_TOWNS_MAJOR) == true)
    {
// preparing the SQL statement [Towns]
      sql =
        wxT("SELECT name, population, type, geom FROM ") +
        MainFrame->GetTowns();
      sql += wxT(" WHERE Intersects(geom, BuildMbr(?, ?, ?, ?))");

      // using the R*Tree Spatial Index
      sql +=
        wxT(" AND ROWID IN (SELECT pkid FROM idx_") + MainFrame->GetTowns() +
        wxT("_geom");
      sql += wxT(" WHERE xmin <= ? AND xmax >= ? AND ymin <= ? AND ymax >= ?)");

      strcpy(xSql, sql.ToUTF8());
      ret = sqlite3_prepare_v2(sqlite, xSql, sql.Len(), &stmt, NULL);
      if (ret != SQLITE_OK)
        {
          wxString err = wxString::FromUTF8(sqlite3_errmsg(sqlite));
          wxMessageBox(wxT("SQL error:") + err, wxT("LibreAtlas"),
                       wxOK | wxICON_ERROR, this);
          goto stop;
        }
// query params binding
      sqlite3_reset(stmt);
      sqlite3_clear_bindings(stmt);
      sqlite3_bind_double(stmt, 1, queryMinX);
      sqlite3_bind_double(stmt, 2, queryMinY);
      sqlite3_bind_double(stmt, 3, queryMaxX);
      sqlite3_bind_double(stmt, 4, queryMaxY);

      // using the R*Tree Spatial Index
      sqlite3_bind_double(stmt, 5, queryMaxX);
      sqlite3_bind_double(stmt, 6, queryMinX);
      sqlite3_bind_double(stmt, 7, queryMaxY);
      sqlite3_bind_double(stmt, 8, queryMinY);

      while (1)
        {
          // scrolling the result set 
          ret = sqlite3_step(stmt);
          if (ret == SQLITE_DONE)
            break;              // end of result set
          if (ret == SQLITE_ROW)
            {
              bool ok = false;
              wxString type =
                wxString::FromUTF8((char *) sqlite3_column_text(stmt, 2));
              if (LayerIsVisible(ATLAS_TOWNS_MINOR) == true)
                ok = true;
              else if (LayerIsVisible(ATLAS_TOWNS_MEDIUM) == true)
                {
                  if (MainFrame->IsWorldWideMap() == true)
                    {
                      if (type.Cmp(wxT("PPLC")) == 0
                          || type.Cmp(wxT("PPLA")) == 0)
                        ok = true;
                  } else
                    {
                      if (sqlite3_column_type(stmt, 1) == SQLITE_INTEGER)
                        {
                          if (sqlite3_column_int(stmt, 1) > 10000)
                            ok = true;
                        }
                    }
              } else if (LayerIsVisible(ATLAS_TOWNS_MAJOR) == true)
                {
                  if (MainFrame->IsWorldWideMap() == true)
                    {
                      if (type.Cmp(wxT("PPLC")) == 0)
                        ok = true;
                  } else
                    {
                      if (sqlite3_column_type(stmt, 1) == SQLITE_INTEGER)
                        {
                          if (sqlite3_column_int(stmt, 1) > 100000)
                            ok = true;
                        }
                    }
                }
              if (ok == true)
                {
                  LabelValue =
                    wxString::FromUTF8((char *) sqlite3_column_text(stmt, 0));
                  if (sqlite3_column_type(stmt, 1) == SQLITE_INTEGER)
                    {
                      sprintf(dummy, " (pop. %d)", sqlite3_column_int(stmt, 1));
                      LabelValue += wxString::FromUTF8(dummy);
                    }
                  const void *blob = sqlite3_column_blob(stmt, 3);
                  int blob_size = sqlite3_column_bytes(stmt, 3);
                  gaiaGeomCollPtr geom =
                    gaiaFromSpatiaLiteBlobWkb((const unsigned char *) blob,
                                              blob_size);
                  if (geom != NULL)
                    SetCurrentGeometry(geom);
                }
          } else
            {
              wxString err = wxString::FromUTF8(sqlite3_errmsg(sqlite));
              wxMessageBox(wxT("SQL error:") + err, wxT("LibreAtlas"),
                           wxOK | wxICON_ERROR, this);
              sqlite3_finalize(stmt);
              goto stop;
            }
        }
      sqlite3_finalize(stmt);
      if (LabelValue.Len() > 0)
        {
          ShowMapTip(LabelValue, mouse_x, mouse_y, ATLAS_TOWNS_MAJOR);
          return;
        }
    }

  if (LayerIsVisible(ATLAS_NAMES) == true)
    {
// preparing the SQL statement [Names]
      sql = wxT("SELECT class, name, geom FROM ") + MainFrame->GetNames();
      sql += wxT(" WHERE Intersects(geom, BuildMbr(?, ?, ?, ?))");

      // using the R*Tree Spatial Index
      sql +=
        wxT(" AND ROWID IN (SELECT pkid FROM idx_") +
        MainFrame->GetNames() + wxT("_geom");
      sql += wxT(" WHERE xmin <= ? AND xmax >= ? AND ymin <= ? AND ymax >= ?)");

      strcpy(xSql, sql.ToUTF8());
      ret = sqlite3_prepare_v2(sqlite, xSql, sql.Len(), &stmt, NULL);
      if (ret != SQLITE_OK)
        {
          wxString err = wxString::FromUTF8(sqlite3_errmsg(sqlite));
          wxMessageBox(wxT("SQL error:") + err, wxT("LibreAtlas"),
                       wxOK | wxICON_ERROR, this);
          goto stop;
        }
// query params binding
      sqlite3_reset(stmt);
      sqlite3_clear_bindings(stmt);
      sqlite3_bind_double(stmt, 1, queryMinX);
      sqlite3_bind_double(stmt, 2, queryMinY);
      sqlite3_bind_double(stmt, 3, queryMaxX);
      sqlite3_bind_double(stmt, 4, queryMaxY);

      // using the R*Tree Spatial Index
      sqlite3_bind_double(stmt, 5, queryMaxX);
      sqlite3_bind_double(stmt, 6, queryMinX);
      sqlite3_bind_double(stmt, 7, queryMaxY);
      sqlite3_bind_double(stmt, 8, queryMinY);

      while (1)
        {
          // scrolling the result set 
          ret = sqlite3_step(stmt);
          if (ret == SQLITE_DONE)
            break;              // end of result set
          if (ret == SQLITE_ROW)
            {
              wxString prefix =
                wxString::FromUTF8((char *) sqlite3_column_text(stmt, 0));
              wxString suffix =
                wxString::FromUTF8((char *) sqlite3_column_text(stmt, 1));
              if (suffix.StartsWith(prefix) == true)
                LabelValue = suffix;
              else
                LabelValue = prefix + wxT(": ") + suffix;
              const void *blob = sqlite3_column_blob(stmt, 2);
              int blob_size = sqlite3_column_bytes(stmt, 2);
              gaiaGeomCollPtr geom =
                gaiaFromSpatiaLiteBlobWkb((const unsigned char *) blob,
                                          blob_size);
              if (geom != NULL)
                SetCurrentGeometry(geom);
          } else
            {
              wxString err = wxString::FromUTF8(sqlite3_errmsg(sqlite));
              wxMessageBox(wxT("SQL error:") + err, wxT("LibreAtlas"),
                           wxOK | wxICON_ERROR, this);
              sqlite3_finalize(stmt);
              goto stop;
            }
        }
      sqlite3_finalize(stmt);
      if (LabelValue.Len() > 0)
        {
          ShowMapTip(LabelValue, mouse_x, mouse_y, ATLAS_NAMES);
          return;
        }
    }

  if (MainFrame->IsModeGraticule() == true)
    {
// preparing the SQL statement [GeographicsLines]
      sql =
        wxT("SELECT name, note, geom FROM ") + MainFrame->GetGeographicLines();
      sql += wxT(" WHERE Intersects(geom, BuildMbr(?, ?, ?, ?))");

      // using the R*Tree Spatial Index
      sql +=
        wxT(" AND ROWID IN (SELECT pkid FROM idx_") +
        MainFrame->GetGeographicLines() + wxT("_geom");
      sql += wxT(" WHERE xmin <= ? AND xmax >= ? AND ymin <= ? AND ymax >= ?)");

      strcpy(xSql, sql.ToUTF8());
      ret = sqlite3_prepare_v2(sqlite, xSql, sql.Len(), &stmt, NULL);
      if (ret != SQLITE_OK)
        {
          wxString err = wxString::FromUTF8(sqlite3_errmsg(sqlite));
          wxMessageBox(wxT("SQL error:") + err, wxT("LibreAtlas"),
                       wxOK | wxICON_ERROR, this);
          goto stop;
        }
// query params binding
      sqlite3_reset(stmt);
      sqlite3_clear_bindings(stmt);
      sqlite3_bind_double(stmt, 1, queryMinX);
      sqlite3_bind_double(stmt, 2, queryMinY);
      sqlite3_bind_double(stmt, 3, queryMaxX);
      sqlite3_bind_double(stmt, 4, queryMaxY);

      // using the R*Tree Spatial Index
      sqlite3_bind_double(stmt, 5, queryMaxX);
      sqlite3_bind_double(stmt, 6, queryMinX);
      sqlite3_bind_double(stmt, 7, queryMaxY);
      sqlite3_bind_double(stmt, 8, queryMinY);

      while (1)
        {
          // scrolling the result set 
          ret = sqlite3_step(stmt);
          if (ret == SQLITE_DONE)
            break;              // end of result set
          if (ret == SQLITE_ROW)
            {
              LabelValue =
                wxString::FromUTF8((char *) sqlite3_column_text(stmt, 0));
              LabelValue += wxT(" - ");
              LabelValue +=
                wxString::FromUTF8((char *) sqlite3_column_text(stmt, 1));
              const void *blob = sqlite3_column_blob(stmt, 2);
              int blob_size = sqlite3_column_bytes(stmt, 2);
              gaiaGeomCollPtr geom =
                gaiaFromSpatiaLiteBlobWkb((const unsigned char *) blob,
                                          blob_size);
              if (geom != NULL)
                SetCurrentGeometry(geom);
          } else
            {
              wxString err = wxString::FromUTF8(sqlite3_errmsg(sqlite));
              wxMessageBox(wxT("SQL error:") + err, wxT("LibreAtlas"),
                           wxOK | wxICON_ERROR, this);
              sqlite3_finalize(stmt);
              goto stop;
            }
        }
      sqlite3_finalize(stmt);
      if (LabelValue.Len() > 0)
        {
          ShowMapTip(LabelValue, mouse_x, mouse_y, ATLAS_GEOGRAPHIC_LINES);
          return;
        }
// preparing the SQL statement [Graticules]
      sql =
        wxT("SELECT type, label, class, geom FROM ") +
        MainFrame->GetGraticules();
      sql += wxT(" WHERE Intersects(geom, BuildMbr(?, ?, ?, ?))");

      // using the R*Tree Spatial Index
      sql +=
        wxT(" AND ROWID IN (SELECT pkid FROM idx_") +
        MainFrame->GetGraticules() + wxT("_geom");
      sql += wxT(" WHERE xmin <= ? AND xmax >= ? AND ymin <= ? AND ymax >= ?)");

      strcpy(xSql, sql.ToUTF8());
      ret = sqlite3_prepare_v2(sqlite, xSql, sql.Len(), &stmt, NULL);
      if (ret != SQLITE_OK)
        {
          wxString err = wxString::FromUTF8(sqlite3_errmsg(sqlite));
          wxMessageBox(wxT("SQL error:") + err, wxT("LibreAtlas"),
                       wxOK | wxICON_ERROR, this);
          goto stop;
        }
// query params binding
      sqlite3_reset(stmt);
      sqlite3_clear_bindings(stmt);
      sqlite3_bind_double(stmt, 1, queryMinX);
      sqlite3_bind_double(stmt, 2, queryMinY);
      sqlite3_bind_double(stmt, 3, queryMaxX);
      sqlite3_bind_double(stmt, 4, queryMaxY);

      // using the R*Tree Spatial Index
      sqlite3_bind_double(stmt, 5, queryMaxX);
      sqlite3_bind_double(stmt, 6, queryMinX);
      sqlite3_bind_double(stmt, 7, queryMaxY);
      sqlite3_bind_double(stmt, 8, queryMinY);

      while (1)
        {
          // scrolling the result set 
          ret = sqlite3_step(stmt);
          if (ret == SQLITE_DONE)
            break;              // end of result set
          if (ret == SQLITE_ROW)
            {
              bool ok = false;
              if (Graticules10Degrees() == true)
                {
                  if (sqlite3_column_type(stmt, 2) == SQLITE_NULL)
                    ;
                  else
                    {
                      if (sqlite3_column_int(stmt, 2) == 10)
                        ok = true;
                    }
              } else if (Graticules10Degrees() == true)
                {
                  if (sqlite3_column_type(stmt, 2) == SQLITE_NULL)
                    ;
                  else
                    ok = true;
              } else
                ok = true;
              if (ok == true)
                {
                  LabelValue =
                    wxString::FromUTF8((char *) sqlite3_column_text(stmt, 0));
                  LabelValue += wxT(": ");
                  LabelValue +=
                    wxString::FromUTF8((char *) sqlite3_column_text(stmt, 1));
                  const void *blob = sqlite3_column_blob(stmt, 3);
                  int blob_size = sqlite3_column_bytes(stmt, 3);
                  gaiaGeomCollPtr geom =
                    gaiaFromSpatiaLiteBlobWkb((const unsigned char *) blob,
                                              blob_size);
                  if (geom != NULL)
                    SetCurrentGeometry(geom);
                }
          } else
            {
              wxString err = wxString::FromUTF8(sqlite3_errmsg(sqlite));
              wxMessageBox(wxT("SQL error:") + err, wxT("LibreAtlas"),
                           wxOK | wxICON_ERROR, this);
              sqlite3_finalize(stmt);
              goto stop;
            }
        }
      sqlite3_finalize(stmt);
      if (LabelValue.Len() > 0)
        {
          ShowMapTip(LabelValue, mouse_x, mouse_y, ATLAS_GRATICULES);
          return;
        }
    }

  if (LayerIsVisible(ATLAS_LAKES) == true)
    {
// preparing the SQL statement [Lakes]
      sql = wxT("SELECT name, geom FROM ") + MainFrame->GetLakes();
      sql += wxT(" WHERE Intersects(geom, BuildMbr(?, ?, ?, ?))");

      // using the R*Tree Spatial Index
      sql +=
        wxT(" AND ROWID IN (SELECT pkid FROM idx_") + MainFrame->GetLakes() +
        wxT("_geom");
      sql += wxT(" WHERE xmin <= ? AND xmax >= ? AND ymin <= ? AND ymax >= ?)");

      strcpy(xSql, sql.ToUTF8());
      ret = sqlite3_prepare_v2(sqlite, xSql, sql.Len(), &stmt, NULL);
      if (ret != SQLITE_OK)
        {
          wxString err = wxString::FromUTF8(sqlite3_errmsg(sqlite));
          wxMessageBox(wxT("SQL error:") + err, wxT("LibreAtlas"),
                       wxOK | wxICON_ERROR, this);
          goto stop;
        }
// query params binding
      sqlite3_reset(stmt);
      sqlite3_clear_bindings(stmt);
      sqlite3_bind_double(stmt, 1, queryMinX);
      sqlite3_bind_double(stmt, 2, queryMinY);
      sqlite3_bind_double(stmt, 3, queryMaxX);
      sqlite3_bind_double(stmt, 4, queryMaxY);

      // using the R*Tree Spatial Index
      sqlite3_bind_double(stmt, 5, queryMaxX);
      sqlite3_bind_double(stmt, 6, queryMinX);
      sqlite3_bind_double(stmt, 7, queryMaxY);
      sqlite3_bind_double(stmt, 8, queryMinY);

      while (1)
        {
          // scrolling the result set 
          ret = sqlite3_step(stmt);
          if (ret == SQLITE_DONE)
            break;              // end of result set
          if (ret == SQLITE_ROW)
            {
              if (sqlite3_column_type(stmt, 0) == SQLITE_NULL)
                LabelValue = wxT("Unknown name");
              else
                LabelValue =
                  wxString::FromUTF8((char *) sqlite3_column_text(stmt, 0));
              const void *blob = sqlite3_column_blob(stmt, 1);
              int blob_size = sqlite3_column_bytes(stmt, 1);
              gaiaGeomCollPtr geom =
                gaiaFromSpatiaLiteBlobWkb((const unsigned char *) blob,
                                          blob_size);
              if (geom != NULL)
                SetCurrentGeometry(geom);
          } else
            {
              wxString err = wxString::FromUTF8(sqlite3_errmsg(sqlite));
              wxMessageBox(wxT("SQL error:") + err, wxT("LibreAtlas"),
                           wxOK | wxICON_ERROR, this);
              sqlite3_finalize(stmt);
              goto stop;
            }
        }
      sqlite3_finalize(stmt);
      if (LabelValue.Len() > 0)
        {
          ShowMapTip(LabelValue, mouse_x, mouse_y, ATLAS_LAKES);
          return;
        }
    }

  if (LayerIsVisible(ATLAS_RIVERS) == true)
    {
// preparing the SQL statement [Rivers]
      sql = wxT("SELECT name, geom FROM ") + MainFrame->GetRivers();
      sql += wxT(" WHERE Intersects(geom, BuildMbr(?, ?, ?, ?))");

      // using the R*Tree Spatial Index
      sql +=
        wxT(" AND ROWID IN (SELECT pkid FROM idx_") + MainFrame->GetRivers() +
        wxT("_geom");
      sql += wxT(" WHERE xmin <= ? AND xmax >= ? AND ymin <= ? AND ymax >= ?)");

      strcpy(xSql, sql.ToUTF8());
      ret = sqlite3_prepare_v2(sqlite, xSql, sql.Len(), &stmt, NULL);
      if (ret != SQLITE_OK)
        {
          wxString err = wxString::FromUTF8(sqlite3_errmsg(sqlite));
          wxMessageBox(wxT("SQL error:") + err, wxT("LibreAtlas"),
                       wxOK | wxICON_ERROR, this);
          goto stop;
        }
// query params binding
      sqlite3_reset(stmt);
      sqlite3_clear_bindings(stmt);
      sqlite3_bind_double(stmt, 1, queryMinX);
      sqlite3_bind_double(stmt, 2, queryMinY);
      sqlite3_bind_double(stmt, 3, queryMaxX);
      sqlite3_bind_double(stmt, 4, queryMaxY);

      // using the R*Tree Spatial Index
      sqlite3_bind_double(stmt, 5, queryMaxX);
      sqlite3_bind_double(stmt, 6, queryMinX);
      sqlite3_bind_double(stmt, 7, queryMaxY);
      sqlite3_bind_double(stmt, 8, queryMinY);

      while (1)
        {
          // scrolling the result set 
          ret = sqlite3_step(stmt);
          if (ret == SQLITE_DONE)
            break;              // end of result set
          if (ret == SQLITE_ROW)
            {
              if (sqlite3_column_type(stmt, 0) == SQLITE_NULL)
                LabelValue = wxT("Unknown name");
              else
                LabelValue =
                  wxString::FromUTF8((char *) sqlite3_column_text(stmt, 0));
              const void *blob = sqlite3_column_blob(stmt, 1);
              int blob_size = sqlite3_column_bytes(stmt, 1);
              gaiaGeomCollPtr geom =
                gaiaFromSpatiaLiteBlobWkb((const unsigned char *) blob,
                                          blob_size);
              if (geom != NULL)
                SetCurrentGeometry(geom);
          } else
            {
              wxString err = wxString::FromUTF8(sqlite3_errmsg(sqlite));
              wxMessageBox(wxT("SQL error:") + err, wxT("LibreAtlas"),
                           wxOK | wxICON_ERROR, this);
              sqlite3_finalize(stmt);
              goto stop;
            }
        }
      sqlite3_finalize(stmt);
      if (LabelValue.Len() > 0)
        {
          ShowMapTip(LabelValue, mouse_x, mouse_y, ATLAS_RIVERS);
          return;
        }
    }

  if (LayerIsVisible(ATLAS_RAILWAYS) == true)
    {
// preparing the SQL statement [Railways]
      sql = wxT("SELECT name, geom FROM ") + MainFrame->GetRailways();
      sql += wxT(" WHERE Intersects(geom, BuildMbr(?, ?, ?, ?))");

      // using the R*Tree Spatial Index
      sql +=
        wxT(" AND ROWID IN (SELECT pkid FROM idx_") + MainFrame->GetRailways() +
        wxT("_geom");
      sql += wxT(" WHERE xmin <= ? AND xmax >= ? AND ymin <= ? AND ymax >= ?)");

      strcpy(xSql, sql.ToUTF8());
      ret = sqlite3_prepare_v2(sqlite, xSql, sql.Len(), &stmt, NULL);
      if (ret != SQLITE_OK)
        {
          wxString err = wxString::FromUTF8(sqlite3_errmsg(sqlite));
          wxMessageBox(wxT("SQL error:") + err, wxT("LibreAtlas"),
                       wxOK | wxICON_ERROR, this);
          goto stop;
        }
// query params binding
      sqlite3_reset(stmt);
      sqlite3_clear_bindings(stmt);
      sqlite3_bind_double(stmt, 1, queryMinX);
      sqlite3_bind_double(stmt, 2, queryMinY);
      sqlite3_bind_double(stmt, 3, queryMaxX);
      sqlite3_bind_double(stmt, 4, queryMaxY);

      // using the R*Tree Spatial Index
      sqlite3_bind_double(stmt, 5, queryMaxX);
      sqlite3_bind_double(stmt, 6, queryMinX);
      sqlite3_bind_double(stmt, 7, queryMaxY);
      sqlite3_bind_double(stmt, 8, queryMinY);

      while (1)
        {
          // scrolling the result set 
          ret = sqlite3_step(stmt);
          if (ret == SQLITE_DONE)
            break;              // end of result set
          if (ret == SQLITE_ROW)
            {
              if (sqlite3_column_type(stmt, 0) == SQLITE_NULL)
                LabelValue = wxT("Unknown name");
              else
                LabelValue =
                  wxString::FromUTF8((char *) sqlite3_column_text(stmt, 0));
              const void *blob = sqlite3_column_blob(stmt, 1);
              int blob_size = sqlite3_column_bytes(stmt, 1);
              gaiaGeomCollPtr geom =
                gaiaFromSpatiaLiteBlobWkb((const unsigned char *) blob,
                                          blob_size);
              if (geom != NULL)
                SetCurrentGeometry(geom);
          } else
            {
              wxString err = wxString::FromUTF8(sqlite3_errmsg(sqlite));
              wxMessageBox(wxT("SQL error:") + err, wxT("LibreAtlas"),
                           wxOK | wxICON_ERROR, this);
              sqlite3_finalize(stmt);
              goto stop;
            }
        }
      sqlite3_finalize(stmt);
      if (LabelValue.Len() > 0)
        {
          ShowMapTip(LabelValue, mouse_x, mouse_y, ATLAS_RAILWAYS);
          return;
        }
    }

  if (LayerIsVisible(ATLAS_HIGHWAYS) == true)
    {
// preparing the SQL statement [Highways]
      sql = wxT("SELECT name, geom FROM ") + MainFrame->GetHighways();
      sql += wxT(" WHERE Intersects(geom, BuildMbr(?, ?, ?, ?))");

      // using the R*Tree Spatial Index
      sql +=
        wxT(" AND ROWID IN (SELECT pkid FROM idx_") + MainFrame->GetHighways() +
        wxT("_geom");
      sql += wxT(" WHERE xmin <= ? AND xmax >= ? AND ymin <= ? AND ymax >= ?)");

      strcpy(xSql, sql.ToUTF8());
      ret = sqlite3_prepare_v2(sqlite, xSql, sql.Len(), &stmt, NULL);
      if (ret != SQLITE_OK)
        {
          wxString err = wxString::FromUTF8(sqlite3_errmsg(sqlite));
          wxMessageBox(wxT("SQL error:") + err, wxT("LibreAtlas"),
                       wxOK | wxICON_ERROR, this);
          goto stop;
        }
// query params binding
      sqlite3_reset(stmt);
      sqlite3_clear_bindings(stmt);
      sqlite3_bind_double(stmt, 1, queryMinX);
      sqlite3_bind_double(stmt, 2, queryMinY);
      sqlite3_bind_double(stmt, 3, queryMaxX);
      sqlite3_bind_double(stmt, 4, queryMaxY);

      // using the R*Tree Spatial Index
      sqlite3_bind_double(stmt, 5, queryMaxX);
      sqlite3_bind_double(stmt, 6, queryMinX);
      sqlite3_bind_double(stmt, 7, queryMaxY);
      sqlite3_bind_double(stmt, 8, queryMinY);

      while (1)
        {
          // scrolling the result set 
          ret = sqlite3_step(stmt);
          if (ret == SQLITE_DONE)
            break;              // end of result set
          if (ret == SQLITE_ROW)
            {
              if (sqlite3_column_type(stmt, 0) == SQLITE_NULL)
                LabelValue = wxT("Unknown name");
              else
                LabelValue =
                  wxString::FromUTF8((char *) sqlite3_column_text(stmt, 0));
              const void *blob = sqlite3_column_blob(stmt, 1);
              int blob_size = sqlite3_column_bytes(stmt, 1);
              gaiaGeomCollPtr geom =
                gaiaFromSpatiaLiteBlobWkb((const unsigned char *) blob,
                                          blob_size);
              if (geom != NULL)
                SetCurrentGeometry(geom);
          } else
            {
              wxString err = wxString::FromUTF8(sqlite3_errmsg(sqlite));
              wxMessageBox(wxT("SQL error:") + err, wxT("LibreAtlas"),
                           wxOK | wxICON_ERROR, this);
              sqlite3_finalize(stmt);
              goto stop;
            }
        }
      sqlite3_finalize(stmt);
      if (LabelValue.Len() > 0)
        {
          ShowMapTip(LabelValue, mouse_x, mouse_y, ATLAS_HIGHWAYS);
          return;
        }
    }

  if (LayerIsVisible(ATLAS_TOWN_AREAS_MAJOR) == true
      || LayerIsVisible(ATLAS_TOWN_AREAS_MEDIUM) == true
      || LayerIsVisible(ATLAS_TOWN_AREAS_MINOR) == true)
    {
// preparing the SQL statement [TownAreas]
      sql =
        wxT("SELECT name, population, type, geom FROM ") +
        MainFrame->GetTownAreas();
      sql += wxT(" WHERE Intersects(geom, BuildMbr(?, ?, ?, ?))");

      // using the R*Tree Spatial Index
      sql +=
        wxT(" AND ROWID IN (SELECT pkid FROM idx_") +
        MainFrame->GetTownAreas() + wxT("_geom");
      sql += wxT(" WHERE xmin <= ? AND xmax >= ? AND ymin <= ? AND ymax >= ?)");

      strcpy(xSql, sql.ToUTF8());
      ret = sqlite3_prepare_v2(sqlite, xSql, sql.Len(), &stmt, NULL);
      if (ret != SQLITE_OK)
        {
          wxString err = wxString::FromUTF8(sqlite3_errmsg(sqlite));
          wxMessageBox(wxT("SQL error:") + err, wxT("LibreAtlas"),
                       wxOK | wxICON_ERROR, this);
          goto stop;
        }
// query params binding
      sqlite3_reset(stmt);
      sqlite3_clear_bindings(stmt);
      sqlite3_bind_double(stmt, 1, queryMinX);
      sqlite3_bind_double(stmt, 2, queryMinY);
      sqlite3_bind_double(stmt, 3, queryMaxX);
      sqlite3_bind_double(stmt, 4, queryMaxY);

      // using the R*Tree Spatial Index
      sqlite3_bind_double(stmt, 5, queryMaxX);
      sqlite3_bind_double(stmt, 6, queryMinX);
      sqlite3_bind_double(stmt, 7, queryMaxY);
      sqlite3_bind_double(stmt, 8, queryMinY);

      while (1)
        {
          // scrolling the result set 
          ret = sqlite3_step(stmt);
          if (ret == SQLITE_DONE)
            break;              // end of result set
          if (ret == SQLITE_ROW)
            {
              bool ok = false;
              wxString type =
                wxString::FromUTF8((char *) sqlite3_column_text(stmt, 2));
              if (LayerIsVisible(ATLAS_TOWN_AREAS_MINOR) == true)
                ok = true;
              else if (LayerIsVisible(ATLAS_TOWN_AREAS_MEDIUM) == true)
                {
                  if (MainFrame->IsWorldWideMap() == true)
                    {
                      if (type.Cmp(wxT("PPLC")) == 0
                          || type.Cmp(wxT("PPLA")) == 0)
                        ok = true;
                  } else
                    {
                      if (sqlite3_column_type(stmt, 1) == SQLITE_INTEGER)
                        {
                          if (sqlite3_column_int(stmt, 1) > 10000)
                            ok = true;
                        }
                    }
              } else if (LayerIsVisible(ATLAS_TOWN_AREAS_MAJOR) == true)
                {
                  if (MainFrame->IsWorldWideMap() == true)
                    {
                      if (type.Cmp(wxT("PPLC")) == 0)
                        ok = true;
                  } else
                    {
                      if (sqlite3_column_type(stmt, 1) == SQLITE_INTEGER)
                        {
                          if (sqlite3_column_int(stmt, 1) > 100000)
                            ok = true;
                        }
                    }
                }
              if (ok == true)
                {
                  LabelValue =
                    wxString::FromUTF8((char *) sqlite3_column_text(stmt, 0));
                  if (sqlite3_column_type(stmt, 1) == SQLITE_INTEGER)
                    {
                      sprintf(dummy, " (pop. %d)", sqlite3_column_int(stmt, 1));
                      LabelValue += wxString::FromUTF8(dummy);
                    }
                  const void *blob = sqlite3_column_blob(stmt, 3);
                  int blob_size = sqlite3_column_bytes(stmt, 3);
                  gaiaGeomCollPtr geom =
                    gaiaFromSpatiaLiteBlobWkb((const unsigned char *) blob,
                                              blob_size);
                  if (geom != NULL)
                    SetCurrentGeometry(geom);
                }
          } else
            {
              wxString err = wxString::FromUTF8(sqlite3_errmsg(sqlite));
              wxMessageBox(wxT("SQL error:") + err, wxT("LibreAtlas"),
                           wxOK | wxICON_ERROR, this);
              sqlite3_finalize(stmt);
              goto stop;
            }
        }
      sqlite3_finalize(stmt);
      if (LabelValue.Len() > 0)
        {
          ShowMapTip(LabelValue, mouse_x, mouse_y, ATLAS_TOWN_AREAS_MAJOR);
          return;
        }
    }

  if (LayerIsVisible(ATLAS_NATURAL_RESERVES) == true)
    {
// preparing the SQL statement [NaturalReserves]
      sql = wxT("SELECT name, geom FROM ") + MainFrame->GetNaturalReserves();
      sql += wxT(" WHERE Intersects(geom, BuildMbr(?, ?, ?, ?))");

      // using the R*Tree Spatial Index
      sql +=
        wxT(" AND ROWID IN (SELECT pkid FROM idx_") +
        MainFrame->GetNaturalReserves() + wxT("_geom");
      sql += wxT(" WHERE xmin <= ? AND xmax >= ? AND ymin <= ? AND ymax >= ?)");

      strcpy(xSql, sql.ToUTF8());
      ret = sqlite3_prepare_v2(sqlite, xSql, sql.Len(), &stmt, NULL);
      if (ret != SQLITE_OK)
        {
          wxString err = wxString::FromUTF8(sqlite3_errmsg(sqlite));
          wxMessageBox(wxT("SQL error:") + err, wxT("LibreAtlas"),
                       wxOK | wxICON_ERROR, this);
          goto stop;
        }
// query params binding
      sqlite3_reset(stmt);
      sqlite3_clear_bindings(stmt);
      sqlite3_bind_double(stmt, 1, queryMinX);
      sqlite3_bind_double(stmt, 2, queryMinY);
      sqlite3_bind_double(stmt, 3, queryMaxX);
      sqlite3_bind_double(stmt, 4, queryMaxY);

      // using the R*Tree Spatial Index
      sqlite3_bind_double(stmt, 5, queryMaxX);
      sqlite3_bind_double(stmt, 6, queryMinX);
      sqlite3_bind_double(stmt, 7, queryMaxY);
      sqlite3_bind_double(stmt, 8, queryMinY);

      minId = 1000000000;
      while (1)
        {
          // scrolling the result set 
          ret = sqlite3_step(stmt);
          if (ret == SQLITE_DONE)
            break;              // end of result set
          if (ret == SQLITE_ROW)
            {
              LabelValue =
                wxString::FromUTF8((char *) sqlite3_column_text(stmt, 0));
              const void *blob = sqlite3_column_blob(stmt, 1);
              int blob_size = sqlite3_column_bytes(stmt, 1);
              gaiaGeomCollPtr geom =
                gaiaFromSpatiaLiteBlobWkb((const unsigned char *) blob,
                                          blob_size);
              if (geom != NULL)
                SetCurrentGeometry(geom);
          } else
            {
              wxString err = wxString::FromUTF8(sqlite3_errmsg(sqlite));
              wxMessageBox(wxT("SQL error:") + err, wxT("LibreAtlas"),
                           wxOK | wxICON_ERROR, this);
              sqlite3_finalize(stmt);
              goto stop;
            }
        }
      sqlite3_finalize(stmt);
      if (LabelValue.Len() > 0)
        {
          ShowMapTip(LabelValue, mouse_x, mouse_y, ATLAS_NATURAL_RESERVES);
          return;
        }
    }

  if (MainFrame->IsPoliticalMap() == false
      && MainFrame->IsModeIdentify() == false)
    {
      if (MainFrame->HasLandAreas() == true)
        {
// preparing the SQL statement [LandAreas]
          sql =
            wxT("SELECT pk_uid, class, name, geom FROM ") +
            MainFrame->GetLandAreas();
          sql += wxT(" WHERE Intersects(geom, BuildMbr(?, ?, ?, ?))");

          // using the R*Tree Spatial Index
          sql +=
            wxT(" AND ROWID IN (SELECT pkid FROM idx_") +
            MainFrame->GetLandAreas() + wxT("_geom");
          sql +=
            wxT(" WHERE xmin <= ? AND xmax >= ? AND ymin <= ? AND ymax >= ?)");

          strcpy(xSql, sql.ToUTF8());
          ret = sqlite3_prepare_v2(sqlite, xSql, sql.Len(), &stmt, NULL);
          if (ret != SQLITE_OK)
            {
              wxString err = wxString::FromUTF8(sqlite3_errmsg(sqlite));
              wxMessageBox(wxT("SQL error:") + err, wxT("LibreAtlas"),
                           wxOK | wxICON_ERROR, this);
              goto stop;
            }
// query params binding
          sqlite3_reset(stmt);
          sqlite3_clear_bindings(stmt);
          sqlite3_bind_double(stmt, 1, queryMinX);
          sqlite3_bind_double(stmt, 2, queryMinY);
          sqlite3_bind_double(stmt, 3, queryMaxX);
          sqlite3_bind_double(stmt, 4, queryMaxY);

          // using the R*Tree Spatial Index
          sqlite3_bind_double(stmt, 5, queryMaxX);
          sqlite3_bind_double(stmt, 6, queryMinX);
          sqlite3_bind_double(stmt, 7, queryMaxY);
          sqlite3_bind_double(stmt, 8, queryMinY);

          minId = 1000000000;
          while (1)
            {
              // scrolling the result set 
              ret = sqlite3_step(stmt);
              if (ret == SQLITE_DONE)
                break;          // end of result set
              if (ret == SQLITE_ROW)
                {
                  sqlite3_int64 id = sqlite3_column_int64(stmt, 0);
                  if (id < minId)
                    {
                      minId = id;
                      LabelValue =
                        wxString::FromUTF8((char *)
                                           sqlite3_column_text(stmt, 1));
                      LabelValue += wxT(": ");
                      LabelValue +=
                        wxString::FromUTF8((char *)
                                           sqlite3_column_text(stmt, 2));
                      const void *blob = sqlite3_column_blob(stmt, 3);
                      int blob_size = sqlite3_column_bytes(stmt, 3);
                      gaiaGeomCollPtr geom =
                        gaiaFromSpatiaLiteBlobWkb((const unsigned char *) blob,
                                                  blob_size);
                      if (geom != NULL)
                        SetCurrentGeometry(geom);
                    }
              } else
                {
                  wxString err = wxString::FromUTF8(sqlite3_errmsg(sqlite));
                  wxMessageBox(wxT("SQL error:") + err, wxT("LibreAtlas"),
                               wxOK | wxICON_ERROR, this);
                  sqlite3_finalize(stmt);
                  goto stop;
                }
            }
          sqlite3_finalize(stmt);
          if (LabelValue.Len() > 0)
            {
              ShowMapTip(LabelValue, mouse_x, mouse_y, ATLAS_LAND_AREAS);
              return;
            }
        }
    }

  if (MainFrame->HasMarineAreas() == true)
    {
// preparing the SQL statement [MarineAreas]
      sql = wxT("SELECT name, geom FROM ") + MainFrame->GetMarineAreas();
      sql += wxT(" WHERE Intersects(geom, BuildMbr(?, ?, ?, ?))");

      // using the R*Tree Spatial Index
      sql +=
        wxT(" AND ROWID IN (SELECT pkid FROM idx_") +
        MainFrame->GetMarineAreas() + wxT("_geom");
      sql += wxT(" WHERE xmin <= ? AND xmax >= ? AND ymin <= ? AND ymax >= ?)");

      strcpy(xSql, sql.ToUTF8());
      ret = sqlite3_prepare_v2(sqlite, xSql, sql.Len(), &stmt, NULL);
      if (ret != SQLITE_OK)
        {
          wxString err = wxString::FromUTF8(sqlite3_errmsg(sqlite));
          wxMessageBox(wxT("SQL error:") + err, wxT("LibreAtlas"),
                       wxOK | wxICON_ERROR, this);
          goto stop;
        }
// query params binding
      sqlite3_reset(stmt);
      sqlite3_clear_bindings(stmt);
      sqlite3_bind_double(stmt, 1, queryMinX);
      sqlite3_bind_double(stmt, 2, queryMinY);
      sqlite3_bind_double(stmt, 3, queryMaxX);
      sqlite3_bind_double(stmt, 4, queryMaxY);

      // using the R*Tree Spatial Index
      sqlite3_bind_double(stmt, 5, queryMaxX);
      sqlite3_bind_double(stmt, 6, queryMinX);
      sqlite3_bind_double(stmt, 7, queryMaxY);
      sqlite3_bind_double(stmt, 8, queryMinY);

      while (1)
        {
          // scrolling the result set 
          ret = sqlite3_step(stmt);
          if (ret == SQLITE_DONE)
            break;              // end of result set
          if (ret == SQLITE_ROW)
            {
              LabelValue =
                wxString::FromUTF8((char *) sqlite3_column_text(stmt, 0));
              const void *blob = sqlite3_column_blob(stmt, 1);
              int blob_size = sqlite3_column_bytes(stmt, 1);
              gaiaGeomCollPtr geom =
                gaiaFromSpatiaLiteBlobWkb((const unsigned char *) blob,
                                          blob_size);
              if (geom != NULL)
                SetCurrentGeometry(geom);
          } else
            {
              wxString err = wxString::FromUTF8(sqlite3_errmsg(sqlite));
              wxMessageBox(wxT("SQL error:") + err, wxT("LibreAtlas"),
                           wxOK | wxICON_ERROR, this);
              sqlite3_finalize(stmt);
              goto stop;
            }
        }
      sqlite3_finalize(stmt);
      if (LabelValue.Len() > 0)
        {
          ShowMapTip(LabelValue, mouse_x, mouse_y, ATLAS_MARINE_AREAS);
          return;
        }
    }

  if (MainFrame->IsPoliticalMap() == true
      || MainFrame->IsModeIdentify() == true)
    {
      if (MainFrame->HasCountries() == true)
        {
// preparing the SQL statement [Countries]
          sql = wxT("SELECT name, geom FROM ") + MainFrame->GetCountries();
          sql += wxT(" WHERE Intersects(geom, BuildMbr(?, ?, ?, ?))");

          // using the R*Tree Spatial Index
          sql +=
            wxT(" AND ROWID IN (SELECT pkid FROM idx_") +
            MainFrame->GetCountries() + wxT("_geom");
          sql +=
            wxT(" WHERE xmin <= ? AND xmax >= ? AND ymin <= ? AND ymax >= ?)");

          strcpy(xSql, sql.ToUTF8());
          ret = sqlite3_prepare_v2(sqlite, xSql, sql.Len(), &stmt, NULL);
          if (ret != SQLITE_OK)
            {
              wxString err = wxString::FromUTF8(sqlite3_errmsg(sqlite));
              wxMessageBox(wxT("SQL error:") + err, wxT("LibreAtlas"),
                           wxOK | wxICON_ERROR, this);
              goto stop;
            }
// query params binding
          sqlite3_reset(stmt);
          sqlite3_clear_bindings(stmt);
          sqlite3_bind_double(stmt, 1, queryMinX);
          sqlite3_bind_double(stmt, 2, queryMinY);
          sqlite3_bind_double(stmt, 3, queryMaxX);
          sqlite3_bind_double(stmt, 4, queryMaxY);

          // using the R*Tree Spatial Index
          sqlite3_bind_double(stmt, 5, queryMaxX);
          sqlite3_bind_double(stmt, 6, queryMinX);
          sqlite3_bind_double(stmt, 7, queryMaxY);
          sqlite3_bind_double(stmt, 8, queryMinY);

          while (1)
            {
              // scrolling the result set 
              ret = sqlite3_step(stmt);
              if (ret == SQLITE_DONE)
                break;          // end of result set
              if (ret == SQLITE_ROW)
                {
                  LabelValue =
                    wxString::FromUTF8((char *) sqlite3_column_text(stmt, 0));
                  const void *blob = sqlite3_column_blob(stmt, 1);
                  int blob_size = sqlite3_column_bytes(stmt, 1);
                  gaiaGeomCollPtr geom =
                    gaiaFromSpatiaLiteBlobWkb((const unsigned char *) blob,
                                              blob_size);
                  if (geom != NULL)
                    SetCurrentGeometry(geom);
              } else
                {
                  wxString err = wxString::FromUTF8(sqlite3_errmsg(sqlite));
                  wxMessageBox(wxT("SQL error:") + err, wxT("LibreAtlas"),
                               wxOK | wxICON_ERROR, this);
                  sqlite3_finalize(stmt);
                  goto stop;
                }
            }
          sqlite3_finalize(stmt);
          if (LabelValue.Len() > 0)
            ShowMapTip(LabelValue, mouse_x, mouse_y, ATLAS_COUNTRIES);
          else
            ClearMapTip();
        }
    }

  if (MainFrame->HasAdminAreas() == true)
    {
// preparing the SQL statement [AdminAres]
      sql =
        wxT("SELECT name, geom FROM ") + MainFrame->GetAdminAreas() +
        wxT(" WHERE ");
      if (LayerIsVisible(ATLAS_ADMIN_AREAS_3) == true)
        sql += wxT("class = '3' ");
      else if (LayerIsVisible(ATLAS_ADMIN_AREAS_2) == true)
        sql += wxT("class = '2' ");
      else if (LayerIsVisible(ATLAS_ADMIN_AREAS_1) == true)
        sql += wxT("class = '1' ");
      sql += wxT("AND Intersects(geom, BuildMbr(?, ?, ?, ?))");

      // using the R*Tree Spatial Index
      sql +=
        wxT(" AND ROWID IN (SELECT pkid FROM idx_") +
        MainFrame->GetAdminAreas() + wxT("_geom");
      sql += wxT(" WHERE xmin <= ? AND xmax >= ? AND ymin <= ? AND ymax >= ?)");

      strcpy(xSql, sql.ToUTF8());
      ret = sqlite3_prepare_v2(sqlite, xSql, sql.Len(), &stmt, NULL);
      if (ret != SQLITE_OK)
        {
          wxString err = wxString::FromUTF8(sqlite3_errmsg(sqlite));
          wxMessageBox(wxT("SQL error:") + err, wxT("LibreAtlas"),
                       wxOK | wxICON_ERROR, this);
          goto stop;
        }
// query params binding
      sqlite3_reset(stmt);
      sqlite3_clear_bindings(stmt);
      sqlite3_bind_double(stmt, 1, queryMinX);
      sqlite3_bind_double(stmt, 2, queryMinY);
      sqlite3_bind_double(stmt, 3, queryMaxX);
      sqlite3_bind_double(stmt, 4, queryMaxY);

      // using the R*Tree Spatial Index
      sqlite3_bind_double(stmt, 5, queryMaxX);
      sqlite3_bind_double(stmt, 6, queryMinX);
      sqlite3_bind_double(stmt, 7, queryMaxY);
      sqlite3_bind_double(stmt, 8, queryMinY);

      while (1)
        {
          // scrolling the result set 
          ret = sqlite3_step(stmt);
          if (ret == SQLITE_DONE)
            break;              // end of result set
          if (ret == SQLITE_ROW)
            {
              LabelValue =
                wxString::FromUTF8((char *) sqlite3_column_text(stmt, 0));
              const void *blob = sqlite3_column_blob(stmt, 1);
              int blob_size = sqlite3_column_bytes(stmt, 1);
              gaiaGeomCollPtr geom =
                gaiaFromSpatiaLiteBlobWkb((const unsigned char *) blob,
                                          blob_size);
              if (geom != NULL)
                SetCurrentGeometry(geom);
          } else
            {
              wxString err = wxString::FromUTF8(sqlite3_errmsg(sqlite));
              wxMessageBox(wxT("SQL error:") + err, wxT("LibreAtlas"),
                           wxOK | wxICON_ERROR, this);
              sqlite3_finalize(stmt);
              goto stop;
            }
        }
      sqlite3_finalize(stmt);
      if (LabelValue.Len() > 0)
        ShowMapTip(LabelValue, mouse_x, mouse_y, ATLAS_ADMIN_AREAS_1);
      else
        ClearMapTip();
    }

stop:
  return;
}

bool MyMapView::IsMapTipShown()
{
//
// tests if some MapTip is currently shown
//
  if (!MapTip)
    return false;
  return true;
}

void MyMapView::ShowMapTip(wxString & text, int x, int y, int category)
{
//
// shows the Map Tip
//
  gaiaGeomCollPtr geom = CurrentGeometry;
  CurrentGeometry = NULL;
  ClearMapTip();
  MapTip = new MyTipWindow(this, text, x, y, category);
  CurrentGeometry = geom;
  // starting the Dynamic timer
  if (!TimerDynamic)
    TimerDynamic = new wxTimer(this, ID_DYNAMIC_TIMER);
  DynamicPhase = 0;
  TimerDynamic->Start(125, wxTIMER_ONE_SHOT);
}

void MyMapView::ClearMapTip()
{
//
// destroys the Map Tip [if it exists]
//
  ResetCurrentGeometry();
  if (TimerDynamic)
    {
      TimerDynamic->Stop();
      delete TimerDynamic;
      TimerDynamic = NULL;
    }
  if (MapTip)
    {
      MapTip->Close();
      if (MapTip)
        MapTip->Destroy();
    }
  MapTip = NULL;
  ResetScreenBitmap();
}

GreatCirclePath::GreatCirclePath(double x1, double y1, double x2, double y2)
{
// constructor
  NumPoints = 3;
  Points = new GreatCirclePoint[NumPoints];
// setting the start point
  GreatCirclePoint *pt = GetPoint(0);
  pt->Set(x1, y1);
// setting the end point
  pt = GetPoint(2);
  pt->Set(x2, y2);
  InterpolatePoints();

  for (int i = 0; i < 9; i++)
    {
      // iteratively expanding Points
      ExpandPoints();
      InterpolatePoints();
    }

  double x = x1;
  double y = y1;
  CrossesDateLine = false;
  for (int i = 1; i < GetNumPoints(); i++)
    {
      // checking if crosses the International Date Line
      pt = GetPoint(i);
      if (x > 90.0 && pt->GetX() < -90.0)
        {
          CrossesDateLine = true;
          BreakIndex = i - 1;
          double min = y;
          double max = pt->GetY();
          if (pt->GetY() < min)
            {
              min = pt->GetY();
              max = y;
            }
          DateLineY = min + (max - min) / 2.0;
        }
      if (x < -90.0 && pt->GetX() > 90.0)
        {
          CrossesDateLine = true;
          BreakIndex = i - 1;
          double min = y;
          double max = pt->GetY();
          if (pt->GetY() < min)
            {
              min = pt->GetY();
              max = y;
            }
          DateLineY = min + (max - min) / 2.0;
        }
      x = pt->GetX();
      y = pt->GetY();
    }
}

GreatCirclePath::~GreatCirclePath()
{
// destructor
  if (Points)
    delete[]Points;
}

void GreatCirclePath::ExpandPoints()
{
// expanding points
  int out = 0;
  int in;
  int oldNum = NumPoints;
  GreatCirclePoint *old = Points;
  NumPoints = oldNum + (oldNum - 1);
  Points = new GreatCirclePoint[NumPoints];
  for (in = 0; in < oldNum; in++)
    {
      // copying points
      Points[out] = old[in];
      out += 2;
    }
  delete[]old;
}

void GreatCirclePath::InterpolatePoints()
{
// interpolating points
  for (int i = 2; i < NumPoints; i += 2)
    ComputePoint(i - 2, i, i - 1);
}

void GreatCirclePath::ComputePoint(int from, int to, int mid)
{
// computing a midpoint
  double pi = 3.14159265358979323846;
  GreatCirclePoint *pt1 = GetPoint(from);
  GreatCirclePoint *pt2 = GetPoint(to);
  GreatCirclePoint *pt3 = GetPoint(mid);
  double x1 = pt1->GetX() * pi / 180.0;
  double y1 = pt1->GetY() * pi / 180.0;
  double x2 = pt2->GetX() * pi / 180.0;
  double y2 = pt2->GetY() * pi / 180.0;
  double dlon = x2 - x1;
// midpoint
  double bx = cos(y2) * cos(dlon);
  double by = cos(y2) * sin(dlon);
  double y =
    atan2(sin(y1) + sin(y2), sqrt((cos(y1) + bx) * (cos(y1) + bx) + by * by));
  double x = x1 + atan2(by, cos(y1) + bx);
  x = fmod(x + 3 * pi, 2 * pi) - pi;  // normalize to -180 / +180
  pt3->Set(x * 180.0 / pi, y * 180 / pi);
}

GreatCirclePoint *GreatCirclePath::GetPoint(int n)
{
// fetching a point pointer by position
  if (n < 0 || n >= GetNumPoints())
    return NULL;
  return Points + n;
}
